package ch.unige.aou.controller;

import java.util.List;
import javax.annotation.PostConstruct;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.DependsOn;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.controller.ModuleController;

import ch.unige.aou.business.ApplicationRoleService;
import ch.unige.aou.business.ContributorRoleService;
import ch.unige.aou.business.DocumentFileTypeService;
import ch.unige.aou.business.EventService;
import ch.unige.aou.business.EventTypeService;
import ch.unige.aou.business.InstitutionService;
import ch.unige.aou.business.LabeledLanguageService;
import ch.unige.aou.business.LicenseGroupService;
import ch.unige.aou.business.LicenseService;
import ch.unige.aou.business.NotificationTypeService;
import ch.unige.aou.business.PersonService;
import ch.unige.aou.business.PublicationService;
import ch.unige.aou.business.PublicationSubSubtypeService;
import ch.unige.aou.business.PublicationSubtypeService;
import ch.unige.aou.business.PublicationTypeService;
import ch.unige.aou.business.RoleService;
import ch.unige.aou.business.ScheduledTaskService;
import ch.unige.aou.business.UserService;
import ch.unige.aou.config.AouProperties;
import ch.unige.aou.model.settings.Person;
import ch.unige.aou.rest.ModuleName;
import ch.unige.aou.rest.UrlPath;

@RestController
@DependsOn("solidifyEventPublisher")
@ConditionalOnProperty(prefix = "aou.module.admin", name = "enable")
@RequestMapping(UrlPath.ADMIN)
public class AdminController extends ModuleController {

  private final LabeledLanguageService labeledLanguageService;
  private final LicenseService licenceService;
  private final LicenseGroupService licenceGroupService;
  private final InstitutionService institutionService;
  private final PersonService personService;
  private final UserService userService;
  private final ApplicationRoleService applicationRoleService;
  private final RoleService roleService;
  private final PublicationService publicationService;
  private final PublicationTypeService publicationTypeService;
  private final PublicationSubtypeService publicationSubtypeService;
  private final PublicationSubSubtypeService publicationSubSubtypeService;
  private final DocumentFileTypeService documentFileTypeService;
  private final ContributorRoleService contributorRoleService;
  private final NotificationTypeService notificationTypeService;
  private final EventTypeService eventTypeService;
  private final EventService eventService;
  private final ScheduledTaskService scheduledTaskService;

  private final List<AouProperties.Test.User> users;
  private final List<AouProperties.ScheduledTaskConfig> scheduledTaskConfigs;
  private final boolean isInit;
  private final boolean isBuilding;
  private final boolean isGenerateData;

  AdminController(AouProperties aouProperties, LabeledLanguageService labeledLanguageService, LicenseService licenceService,
          LicenseGroupService licenceGroupService, InstitutionService institutionService,
          PersonService personService, UserService userService, ApplicationRoleService applicationRoleService,
          PublicationTypeService publicationTypeService, PublicationService publicationService,
          PublicationSubtypeService publicationSubtypeService, RoleService roleService, DocumentFileTypeService documentFileTypeService,
          ContributorRoleService contributorRoleService, NotificationTypeService notificationTypeService, EventTypeService eventTypeService,
          EventService eventService, PublicationSubSubtypeService publicationSubSubtypeService, ScheduledTaskService scheduledTaskService) {
    super(ModuleName.ADMIN);

    this.labeledLanguageService = labeledLanguageService;
    this.licenceService = licenceService;
    this.licenceGroupService = licenceGroupService;
    this.institutionService = institutionService;
    this.personService = personService;
    this.userService = userService;
    this.applicationRoleService = applicationRoleService;
    this.publicationTypeService = publicationTypeService;
    this.publicationSubtypeService = publicationSubtypeService;
    this.publicationService = publicationService;
    this.publicationSubSubtypeService = publicationSubSubtypeService;
    this.roleService = roleService;
    this.documentFileTypeService = documentFileTypeService;
    this.contributorRoleService = contributorRoleService;
    this.notificationTypeService = notificationTypeService;
    this.eventTypeService = eventTypeService;
    this.eventService = eventService;
    this.scheduledTaskService = scheduledTaskService;

    this.users = aouProperties.getTest().getUsers();
    this.scheduledTaskConfigs = aouProperties.getScheduledTasks();
    this.isInit = aouProperties.getData().isInit();
    this.isBuilding = aouProperties.getTest().isBuilding();
    this.isGenerateData = aouProperties.getTest().isGenerateData();
  }

  @PostConstruct
  public void initDefaultApplicationData() {
    // Initialize required data
    this.labeledLanguageService.initDefaultData();
    this.applicationRoleService.initDefaultData();
    this.roleService.initDefaultData();
    this.eventTypeService.initDefaultData();
    this.notificationTypeService.initDefaultData();
    this.publicationTypeService.initDefaultData();
    this.publicationSubtypeService.initDefaultData();
    this.publicationSubSubtypeService.initDefaultData();

    // Initialize optional data
    if (this.isInit) {
      this.institutionService.initDefaultData();
      this.contributorRoleService.initDefaultData();
      for (AouProperties.ScheduledTaskConfig scheduledTaskConfig : this.scheduledTaskConfigs) {
        this.scheduledTaskService.createScheduledTaskFromConfig(scheduledTaskConfig);
      }
      this.licenceGroupService.initDefaultData();
    }

    if (!this.isBuilding) {
      this.documentFileTypeService.initDefaultData();
      this.licenceService.initDefaultData();
    }

    // Initialize data for documentation generation
    if (this.isBuilding) {
      for (final AouProperties.Test.User testUser : this.users) {
        final Person testPerson = this.personService.initDefaultData(testUser);
        this.userService.createTestUser(testUser, testPerson);
      }
    }

    // Initialize test data
    if (this.isGenerateData) {
      int index = 0;
      for (final Person testPerson : this.personService.findTestPeople()) {
        if (index == 0) {
          this.publicationService.initDefaultData(testPerson);
          this.eventService.initDefaultData(testPerson);
        }
        index++;
      }
    }
  }
}
