package ch.unige.aou.controller.admin;

import java.util.List;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.config.SolidifyEventPublisher;
import ch.unige.solidify.controller.AssociationController;
import ch.unige.solidify.rest.RestCollection;

import ch.unige.aou.business.UserService;
import ch.unige.aou.config.AouProperties;
import ch.unige.aou.controller.AdminController;
import ch.unige.aou.message.EmailMessage;
import ch.unige.aou.message.EmailMessage.EmailTemplate;
import ch.unige.aou.model.notification.NotificationType;
import ch.unige.aou.model.settings.Person;
import ch.unige.aou.rest.ResourceName;
import ch.unige.aou.rest.UrlPath;
import ch.unige.aou.security.CurrentUserPermissions;

@CurrentUserPermissions
@RestController
@ConditionalOnBean(AdminController.class)
@RequestMapping(UrlPath.ADMIN_PEOPLE + SolidifyConstants.URL_PARENT_ID + ResourceName.NOTIFICATION_TYPES)
public class PersonNotificationTypeController extends AssociationController<Person, NotificationType> {

  private final UserService userService;
  private boolean sendEmailOnNotificationTypesSubscriptionChange;

  public PersonNotificationTypeController(AouProperties aouProperties, UserService userService) {
    this.userService = userService;
    this.sendEmailOnNotificationTypesSubscriptionChange = aouProperties.getParameters().isEmailOnNotificationTypesSubscriptionChangeEnabled();
  }

  @Override
  public HttpEntity<RestCollection<NotificationType>> list(@PathVariable String parentid, @ModelAttribute NotificationType filterItem,
          Pageable pageable) {
    return super.list(parentid, filterItem, pageable);
  }

  @Override
  public HttpEntity<NotificationType> get(@PathVariable String parentid, @PathVariable String id) {
    return super.get(parentid, id);
  }

  @Override
  public HttpEntity<List<NotificationType>> create(@PathVariable String parentid, @RequestBody String[] ids) {
    HttpEntity<List<NotificationType>> result = super.create(parentid, ids);
    this.sendEmailMessage(EmailTemplate.NEW_SUBSCRIPTION, ids);
    return result;
  }

  @Override
  public HttpStatus delete(@PathVariable String parentid, @PathVariable String id) {
    HttpStatus result = super.delete(parentid, id);
    this.sendEmailMessage(EmailTemplate.REMOVE_SUBSCRIPTION, new String[] { id });
    return result;
  }

  @Override
  public HttpStatus deleteList(@PathVariable String parentid, @RequestBody String[] ids) {
    HttpStatus result = super.deleteList(parentid, ids);
    this.sendEmailMessage(EmailTemplate.REMOVE_SUBSCRIPTION, ids);
    return result;
  }

  @Override
  protected String getParentFieldName() {
    return "subscribers";
  }

  @Override
  protected boolean addChildOnParent(Person person, NotificationType notificationType) {
    return person.addNotificationType(notificationType);
  }

  @Override
  protected boolean removeChildFromParent(Person person, NotificationType notificationType) {
    return person.removeNotificationType(notificationType);
  }

  private void sendEmailMessage(EmailTemplate template, String ids[]) {
    if (this.sendEmailOnNotificationTypesSubscriptionChange) {
      SolidifyEventPublisher.getPublisher().publishEvent(new EmailMessage(this.userService.getCurrentUser().getEmail(), template, ids));
    }
  }
}
