package ch.unige.aou.controller.admin;

import java.util.List;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.controller.AssociationController;
import ch.unige.solidify.rest.RestCollection;

import ch.unige.aou.business.StructureService;
import ch.unige.aou.controller.AdminController;
import ch.unige.aou.model.settings.Person;
import ch.unige.aou.model.settings.Structure;
import ch.unige.aou.rest.ResourceName;
import ch.unige.aou.rest.UrlPath;
import ch.unige.aou.security.CurrentUserPermissions;

@CurrentUserPermissions
@RestController
@ConditionalOnBean(AdminController.class)
@RequestMapping(UrlPath.ADMIN_PEOPLE + SolidifyConstants.URL_PARENT_ID + ResourceName.STRUCTURES)
public class PersonStructureController extends AssociationController<Person, Structure> {
  @Override
  public HttpEntity<Structure> get(@PathVariable String parentid, @PathVariable String id) {
    return super.get(parentid, id);
  }

  @Override
  public HttpEntity<RestCollection<Structure>> list(@PathVariable String parentid, @ModelAttribute Structure structure, Pageable pageable) {
    return super.list(parentid, structure, pageable);
  }

  @Override
  public HttpEntity<List<Structure>> create(@PathVariable String parentid, @RequestBody String[] structureIds) {
    return super.create(parentid, structureIds);
  }

  @Override
  public HttpStatus delete(@PathVariable String parentid, @PathVariable String id) {
    return super.delete(parentid, id);
  }

  @Override
  public HttpStatus deleteList(@PathVariable String parentid, @RequestBody String[] ids) {
    return super.deleteList(parentid, ids);
  }

  @Override
  protected String getParentFieldName() {
    return "people";
  }

  @Override
  protected boolean addChildOnParent(Person person, Structure structure) {
    ((StructureService) this.subResourceService).duplicatesInStructures(person, structure);
    return person.addStructure(structure);
  }

  @Override
  protected boolean removeChildFromParent(Person person, Structure structure) {
    return person.removeStructure(structure);
  }

}
