package ch.unige.aou.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Service;

import ch.unige.aou.controller.AdminController;
import ch.unige.aou.message.DocumentFileMessage;

@Service
@ConditionalOnBean(AdminController.class)
public class DocumentFileJmsListener {

  private static final Logger log = LoggerFactory.getLogger(DocumentFileJmsListener.class);

  private DocumentFileProcessingService documentFileProcessingService;
  private DocumentFileJmsEmitter documentFileJmsEmitter;

  public DocumentFileJmsListener(DocumentFileProcessingService documentFileProcessingService, DocumentFileJmsEmitter documentFileJmsEmitter) {
    this.documentFileProcessingService = documentFileProcessingService;
    this.documentFileJmsEmitter = documentFileJmsEmitter;
  }

  @JmsListener(destination = "${aou.queue.documentFiles}")
  public void receiveDocumentFileMessage(DocumentFileMessage documentFileMessage) {
    log.info("received DocumentFileMessage: " + documentFileMessage.toString());

    final boolean mustSendNewMessage = this.documentFileProcessingService.processDocumentFile(documentFileMessage);

    /*
     * Sending a new message is done outside processDocumentFile() to ensure Transaction is committed
     * when the message is sent
     */
    if (mustSendNewMessage) {
      log.info("----> DocumentFileJmsListener will send a DocumentFileMessage for DocumentFile '{}'", documentFileMessage.getResId());
      this.documentFileJmsEmitter.sendDocumentFileMessageToQueue(new DocumentFileMessage(documentFileMessage.getResId()));
    }
  }
}
