package ch.unige.aou.service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.core.env.Environment;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.web.client.RestTemplate;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.Resource;
import ch.unige.solidify.rest.RestCollectionPage;
import ch.unige.solidify.util.AbstractRestClientTool;
import ch.unige.solidify.util.CollectionTool;

public abstract class CompositionResourceClientService<V extends Resource, T extends Resource>
        extends ResourceBaseClientService<T> implements CompositionResourceClientServiceInterface<V, T> {

  public CompositionResourceClientService(String resourcePath, Class<T> resourceClass, String module, AbstractRestClientTool restClientTool,
          Environment env) {
    super(resourcePath, resourceClass, module, restClientTool, env);
  }

  @Override
  public T findOne(String parentId, String resId) {
    final String url = this.getBaseUrl() + this.resourcePath + SolidifyConstants.URL_ID;
    final Map<String, String> params = new HashMap<>();
    params.put(SolidifyConstants.URL_PARENT_ID_FIELD, parentId);
    params.put(SolidifyConstants.URL_ID_FIELD, resId);
    return this.getObjectFromUrl(url, params);
  }

  protected List<T> findAll(String parentId, Class<T> itemClass) {
    final String url = this.getBaseUrl() + this.resourcePath + "?size=" + RestCollectionPage.MAX_SIZE_PAGE;
    final Map<String, String> params = new HashMap<>();
    params.put(SolidifyConstants.URL_PARENT_ID_FIELD, parentId);
    final RestTemplate restTemplate = this.restClientTool.getClient();
    final String jsonString = restTemplate.getForObject(url, String.class, params);
    return CollectionTool.getList(jsonString, itemClass);
  }

  @Override
  public T create(String parentId, T resource) {
    final String url = this.getBaseUrl() + this.resourcePath;
    final Map<String, String> params = new HashMap<>();
    params.put(SolidifyConstants.URL_PARENT_ID_FIELD, parentId);
    final RestTemplate restTemplate = this.restClientTool.getClient();
    return restTemplate.postForObject(url, resource, this.getResourceClass(), params);
  }

  @Override
  public T update(String parentId, String resId, T resource) {
    final String url = this.getBaseUrl() + this.resourcePath + SolidifyConstants.URL_ID;
    final Map<String, String> params = new HashMap<>();
    params.put(SolidifyConstants.URL_PARENT_ID_FIELD, parentId);
    params.put(SolidifyConstants.URL_ID_FIELD, resId);
    final RestTemplate restTemplate = this.restClientTool.getClient();
    return restTemplate.exchange(url, HttpMethod.PATCH, new HttpEntity<>(resource), this.getResourceClass(), params).getBody();
  }

  @Override
  public void delete(String parentId, String resId) {
    String url = this.getBaseUrl() + this.resourcePath;
    url = url.replace(SolidifyConstants.URL_PARENT_ID, SolidifyConstants.URL_SEP + parentId + SolidifyConstants.URL_SEP);
    url += SolidifyConstants.URL_SEP + resId;

    final RestTemplate restTemplate = this.restClientTool.getClient();
    restTemplate.delete(url);
  }
}
