package ch.unige.aou.service.admin;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.RestCollectionPage;
import ch.unige.solidify.util.AbstractRestClientTool;
import ch.unige.solidify.util.CollectionTool;

import ch.unige.aou.model.notification.Notification;
import ch.unige.aou.model.settings.Person;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.ModuleName;
import ch.unige.aou.rest.ResourceName;
import ch.unige.aou.service.CompositionResourceClientService;

@Service
public class PersonNotificationClientService extends CompositionResourceClientService<Person, Notification>
        implements PersonNotificationClientServiceInterface {

  private static final String PEOPLE_NOTIFICATION_TYPES_PATH =
          SolidifyConstants.URL_SEP + ResourceName.PEOPLE + SolidifyConstants.URL_PARENT_ID + ResourceName.NOTIFICATIONS;

  public PersonNotificationClientService(AbstractRestClientTool restClientTool, Environment env) {
    super(PEOPLE_NOTIFICATION_TYPES_PATH, Notification.class, ModuleName.ADMIN, restClientTool, env);
  }

  @Override
  public List<Notification> findAll(String personId) {
    return this.findAll(personId, Notification.class);
  }

  public List<Notification> findAll(String personId, Map<String, String> queryParameters) {
    if (!queryParameters.containsKey("size")) {
      queryParameters.put("size", String.valueOf(RestCollectionPage.MAX_SIZE_PAGE));
    }
    String url = this.getBaseUrl() + this.resourcePath + "?";
    for (final Map.Entry<String, String> entry : queryParameters.entrySet()) {
      url += "&" + entry.getKey() + "=" + entry.getValue();
    }

    final Map<String, String> params = new HashMap<>();
    params.put(SolidifyConstants.URL_PARENT_ID_FIELD, personId);
    final RestTemplate restTemplate = this.restClientTool.getClient();
    final String jsonString = restTemplate.getForObject(url, String.class, params);
    return CollectionTool.getList(jsonString, Notification.class);
  }

  public Notification markNotificationAsRead(String personId, String notificationId) {
    final String url = this.getBaseUrl() + this.resourcePath + SolidifyConstants.URL_ID + SolidifyConstants.URL_SEP + AouActionName.SET_READ;
    final Map<String, String> params = new HashMap<>();
    params.put(SolidifyConstants.URL_PARENT_ID_FIELD, personId);
    params.put(SolidifyConstants.URL_ID_FIELD, notificationId);
    final RestTemplate restTemplate = this.restClientTool.getClient();
    return restTemplate.postForObject(url, null, Notification.class, params);
  }
}
