package ch.unige.aou.service.admin;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.util.AbstractRestClientTool;

import ch.unige.aou.model.publication.PublicationType;
import ch.unige.aou.rest.ModuleName;
import ch.unige.aou.rest.ResourceName;
import ch.unige.aou.service.ResourceClientService;

@Service
public class PublicationTypeClientService extends ResourceClientService<PublicationType> implements PublicationTypeClientServiceInterface {

  private static final String PUBLICATION_TYPES_PATH = SolidifyConstants.URL_SEP + ResourceName.PUBLICATION_TYPES;

  public PublicationTypeClientService(AbstractRestClientTool restClientTool, Environment env) {
    super(PUBLICATION_TYPES_PATH, PublicationType.class, ModuleName.ADMIN, restClientTool, env);
  }

  @Override
  public List<PublicationType> searchByProperties(Map<String, String> properties) {
    return this.searchByProperties(properties, PublicationType.class);
  }

  @Override
  public List<PublicationType> findAll() {
    return this.findAll(PublicationType.class);
  }

  @Override
  public PublicationType findByName(String name) {
    final Map<String, String> properties = new HashMap<>();
    properties.put("name", name);
    List<PublicationType> publicationTypes = this.searchByProperties(properties);
    if (publicationTypes.size() == 1) {
      return publicationTypes.get(0);
    } else if (publicationTypes.isEmpty()) {
      throw new SolidifyRuntimeException("No publication type with name '" + name + "' found");
    } else {
      throw new SolidifyRuntimeException("Too many publication type with name '" + name + "' found");
    }
  }
}
