package ch.unige.aou.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import controller.DataMgmtController;

import ch.unige.solidify.service.MessageService;

import ch.unige.aou.business.PublicationService;
import ch.unige.aou.model.index.PublicationIndexEntry;
import ch.unige.aou.model.publication.Publication;
import ch.unige.aou.service.elasticsearch.PublicationIndexingService;

@Service
@ConditionalOnBean(DataMgmtController.class)
public class IndexingProcessingService extends AouService {

  private static final Logger log = LoggerFactory.getLogger(IndexingProcessingService.class);

  private static final String PUBLICATIONS_INDEX_NAME = "publications";

  private final PublicationService publicationService;
  private final PublicationIndexingService publicationIndexingService;

  public IndexingProcessingService(MessageService messageService, PublicationService publicationService,
          PublicationIndexingService publicationIndexingService) {
    super(messageService);
    this.publicationService = publicationService;
    this.publicationIndexingService = publicationIndexingService;
  }

  @Transactional
  public void indexPublication(String publicationId) {

    Publication publication = this.publicationService.findOne(publicationId);
    PublicationIndexEntry publicationIndexEntry = this.getPublicationIndexEntry(publication);
    this.publicationIndexingService.save(PUBLICATIONS_INDEX_NAME, publicationIndexEntry);
    log.info("Publication '{}' ({}) indexed", publication.getResId(), publication.getTitle());
  }

  private PublicationIndexEntry getPublicationIndexEntry(Publication publication) {
    PublicationIndexEntry publicationIndexEntry = new PublicationIndexEntry();
    publicationIndexEntry.setMetadata(publication.getMetadata());
    publicationIndexEntry.setIndex(PUBLICATIONS_INDEX_NAME);
    return publicationIndexEntry;
  }
}
