package ch.unige.aou.model.display;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.List;

import org.slf4j.LoggerFactory;
import org.springframework.hateoas.server.mvc.WebMvcLinkBuilder;

import ch.unige.solidify.rest.ResourceBase;
import ch.unige.solidify.util.ReflectionTool;

public interface ExtendedResourceInterface<V> {
  void addLinks(WebMvcLinkBuilder linkBuilder, boolean mainRes, boolean subResOnly);

  V getItem();

  List<V> getItems();

  String getResId();

  default <T, V> void mapResourceToExtendedResource(T extendedResource, V resource) {
    final List<Class> classes = ReflectionTool.getParentClasses(resource, ResourceBase.class);
    for (final Class<?> clz : classes) {
      final Field[] fields = clz.getDeclaredFields();
      for (final Field fieldToCheck : fields) {

        fieldToCheck.setAccessible(true);

        try {

          final Field extendedResourceField = ReflectionTool.getField(extendedResource, fieldToCheck.getName());
          if (!Modifier.isFinal(extendedResourceField.getModifiers())) {
            final Object fieldValue = fieldToCheck.get(resource);
            fieldToCheck.set(extendedResource, fieldValue);
          }

        } catch (final IllegalAccessException e) {
          LoggerFactory.getLogger(ExtendedResourceInterface.class)
                  .error("Could not convert " + resource.getClass().getName() + " to " + extendedResource.getClass().getName(), e);
        } catch (final NoSuchFieldException e) {
          LoggerFactory.getLogger(ExtendedResourceInterface.class).error("Field " + fieldToCheck.getName() + " not found", e);
        }

        fieldToCheck.setAccessible(false);
      }
    }
  }

  void setItem(V item);

  void setItems(List<V> items);
}
