package ch.unige.aou.model.notification;

import java.time.OffsetDateTime;
import java.util.Objects;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.ManyToOne;
import javax.validation.constraints.NotNull;

import com.fasterxml.jackson.annotation.JsonFormat;

import ch.unige.solidify.util.StringTool;

import ch.unige.aou.model.AouResourceNormalized;
import ch.unige.aou.model.settings.Person;

@Entity
public class Notification extends AouResourceNormalized {

  @NotNull
  @ManyToOne
  private Person recipient;

  @NotNull
  private String message;

  @NotNull
  @ManyToOne
  private NotificationType notificationType;

  @NotNull
  @ManyToOne
  private Event event;

  @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = StringTool.DATE_TIME_FORMAT)
  @Column(length = 3)
  private OffsetDateTime readTime;

  @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = StringTool.DATE_TIME_FORMAT)
  @Column(length = 3)
  private OffsetDateTime sentTime;

  @Override
  public boolean equals(final Object o) {
    if (this == o) {
      return true;
    }
    if (!(o instanceof Notification)) {
      return false;
    }
    if (!super.equals(o)) {
      return false;
    }
    final Notification that = (Notification) o;
    return this.notificationType == that.notificationType &&
            Objects.equals(this.recipient, that.recipient) &&
            Objects.equals(this.message, that.message);
  }

  @Override
  public int hashCode() {
    return Objects.hash(super.hashCode(), this.notificationType, this.recipient, this.message);
  }

  public Person getRecipient() {
    return this.recipient;
  }

  public void setRecipient(Person recipient) {
    this.recipient = recipient;
  }

  public String getMessage() {
    return this.message;
  }

  public void setMessage(String message) {
    this.message = message;
  }

  public NotificationType getNotificationType() {
    return this.notificationType;
  }

  public void setNotificationType(NotificationType notificationType) {
    this.notificationType = notificationType;
  }

  public OffsetDateTime getReadTime() {
    return this.readTime;
  }

  public void setReadTime(OffsetDateTime readTime) {
    this.readTime = readTime;
  }

  public OffsetDateTime getSentTime() {
    return this.sentTime;
  }

  public void setSentTime(OffsetDateTime sentTime) {
    this.sentTime = sentTime;
  }

  public Event getEvent() {
    return this.event;
  }

  public void setEvent(Event event) {
    this.event = event;
  }
}
