package ch.unige.aou.model.publication;

import javax.persistence.Entity;
import javax.persistence.ManyToOne;
import javax.persistence.PrePersist;
import javax.persistence.PreUpdate;
import javax.persistence.Transient;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.aou.AouConstants;
import ch.unige.aou.model.AouResourceNormalized;
import ch.unige.aou.model.PublicationAwareResource;
import ch.unige.aou.model.settings.Person;

@Entity
public class Comment extends AouResourceNormalized implements PublicationAwareResource {

  @NotNull
  @Size(min = 1, max = AouConstants.DB_LARGE_STRING_LENGTH)
  private String text;

  @NotNull
  @ManyToOne(targetEntity = Publication.class)
  @JsonIgnore
  private Publication publication;

  @ManyToOne(targetEntity = Person.class)
  private Person person;

  @NotNull
  private Boolean onlyForValidators;

  @Transient
  @JsonIgnore
  public Boolean hasOnlyForValidatorValue() {
    return (this.onlyForValidators != null);
  }

  public boolean isOnlyForValidators() {
    if (this.onlyForValidators != null) {
      return this.onlyForValidators;
    } else {
      return Boolean.TRUE;
    }
  }

  public String getText() {
    return this.text;
  }

  public Publication getPublication() {
    return this.publication;
  }

  public Person getPerson() {
    return this.person;
  }

  public void setText(String text) {
    this.text = text;
  }

  public void setPublication(Publication publication) {
    this.publication = publication;
  }

  public void setPerson(Person person) {
    this.person = person;
  }

  public void setOnlyForValidators(boolean onlyForValidators) {
    this.onlyForValidators = onlyForValidators;
  }

  @JsonIgnore
  @Override
  public String getPublicationId() {
    return this.publication.getResId();
  }

  @JsonIgnore
  @Override
  public String getCreatorId() {
    return this.person.getResId();
  }

  @Override
  @PrePersist
  @PreUpdate
  public void init() {
    if (this.onlyForValidators == null) {
      this.onlyForValidators = true;
    }
  }
}
