package ch.unige.aou.model.settings;

import javax.persistence.Column;
import javax.persistence.Embeddable;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.MappedSuperclass;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import com.fasterxml.jackson.annotation.JsonProperty;

import ch.unige.solidify.SolidifyConstants;

import ch.unige.aou.AouConstants;

@Embeddable
@MappedSuperclass
public class Label {

  @NotNull
  @Size(min = 1, max = AouConstants.DB_LONG_STRING_LENGTH)
  @Column(nullable = false)
  private String text;

  @NotNull
  @ManyToOne
  @JoinColumn(name = AouConstants.DB_LANGUAGE_ID, referencedColumnName = SolidifyConstants.DB_RES_ID, nullable = false)
  @JsonProperty(access = JsonProperty.Access.WRITE_ONLY)
  private LabeledLanguage language;

  public Label() {
  }

  public Label(String text, LabeledLanguage language) {
    this.text = text;
    this.language = language;
  }

  public String getText() {
    return this.text;
  }

  public LabeledLanguage getLanguage() {
    return this.language;
  }

  public void setText(String text) {
    this.text = text;
  }

  public void setLanguage(LabeledLanguage language) {
    this.language = language;
  }

  public String getLanguageId() {
    if (this.language != null) {
      return this.language.getResId();
    }
    return null;
  }

  public String getLanguageCode() {
    if (this.language != null) {
      return this.language.getCode();
    }
    return null;
  }
}
