package ch.unige.aou.model.settings;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import javax.persistence.CollectionTable;
import javax.persistence.Column;
import javax.persistence.ElementCollection;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import ch.unige.solidify.model.Language;

import ch.unige.aou.AouConstants;
import ch.unige.aou.model.AouResourceNormalized;

/**
 * Language representation names based in ISO 639-2
 */
@Entity
@Table(name = "language")
public class LabeledLanguage extends AouResourceNormalized {

  @Column(unique = true, nullable = false)
  private String code;

  /**
   * Property allowing to sort languages in lists
   */
  @Column(nullable = false)
  private Integer sortValue;

  @ElementCollection
  @Column(name = AouConstants.DB_LANGUAGE_ENTITY_ID)
  @CollectionTable(name = "language_labels",
                   joinColumns = { @JoinColumn(name = AouConstants.DB_LANGUAGE_ENTITY_ID) },
                   uniqueConstraints = @UniqueConstraint(columnNames = { AouConstants.DB_LANGUAGE_ENTITY_ID, AouConstants.DB_LANGUAGE_ID }))
  private List<Label> labels = new ArrayList<>();

  /****************************************************************/

  public String getCode() {
    return this.code;
  }

  public void setCode(String code) {
    this.code = code;
  }

  public Integer getSortValue() {
    return this.sortValue;
  }

  public void setSortValue(Integer sortValue) {
    this.sortValue = sortValue;
  }

  public List<Label> getLabels() {
    return this.labels;
  }

  public void setLabels(List<Label> labels) {
    this.labels = labels;
  }

  /****************************************************************/

  @Override
  public boolean equals(final Object other) {
    if (this == other) {
      return true;
    }
    if (!(other instanceof Language)) {
      return false;
    }
    final Language castOther = (Language) other;
    return Objects.equals(this.getCode(), castOther.getCode());
  }

  @Override
  public int hashCode() {
    return Objects.hash(this.getCode());
  }
}
