package ch.unige.aou.model.settings;

import java.util.ArrayList;
import java.util.List;
import javax.persistence.CollectionTable;
import javax.persistence.Column;
import javax.persistence.ElementCollection;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.UniqueConstraint;
import javax.validation.constraints.NotNull;

import ch.unige.aou.AouConstants;
import ch.unige.aou.model.AouResourceNormalized;
import ch.unige.aou.model.Sortable;

/**
 * Structure based on open licenses. http://licenses.opendefinition.org
 */
@Entity
public class LicenseGroup extends AouResourceNormalized implements Sortable {

  @NotNull
  @Column(unique = true)
  private String name;

  /**
   * Property allowing to sort licenses inside groups
   */
  @Column(nullable = false)
  private Integer sortValue;

  @Column(nullable = false)
  private Boolean visible;

  @ElementCollection
  @Column(name = AouConstants.DB_LICENSE_GROUP_ID)
  @CollectionTable(joinColumns = { @JoinColumn(name = AouConstants.DB_LICENSE_GROUP_ID) },
                   uniqueConstraints = @UniqueConstraint(columnNames = { AouConstants.DB_LICENSE_GROUP_ID, AouConstants.DB_LANGUAGE_ID }))
  private List<Label> labels = new ArrayList<>();

  /***********************************************************/

  public String getName() {
    return this.name;
  }

  @Override
  public Integer getSortValue() {
    return this.sortValue;
  }

  public Boolean getVisible() {
    return this.visible;
  }

  public List<Label> getLabels() {
    return this.labels;
  }

  /***********************************************************/

  public void setName(String name) {
    this.name = name;
  }

  @Override
  public void setSortValue(Integer sortValue) {
    this.sortValue = sortValue;
  }

  public void setVisible(Boolean visible) {
    this.visible = visible;
  }

  public void setLabels(List<Label> labels) {
    this.labels = labels;
  }

  /***********************************************************/

  @Override
  public void init() {
    if (this.visible == null) {
      this.visible = true;
    }
  }
}
