package ch.unige.aou.model.xml.deposit.v1.deserializer;

import java.io.IOException;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.ObjectCodec;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonNode;

import ch.unige.aou.model.xml.deposit.AouStdDeserializer;
import ch.unige.aou.model.xml.deposit.v1.ContainerIdentifiers;
import ch.unige.aou.model.xml.deposit.v1.Dois;

public class ContainerIdentifiersDeserializer extends AouStdDeserializer<ContainerIdentifiers> {

  private static final long serialVersionUID = 659865L;

  @Override
  public ContainerIdentifiers deserialize(JsonParser parser, DeserializationContext context) throws IOException, JsonProcessingException {

    ObjectCodec codec = parser.getCodec();
    JsonNode jsonNode = codec.readTree(parser);

    boolean hasValue = false;
    ContainerIdentifiers containerIdentifiers = new ContainerIdentifiers();

    JsonNode doisNode = jsonNode.get("dois");
    if (doisNode != null) {
      Dois dois = codec.treeToValue(doisNode, Dois.class);
      if (dois != null && !dois.getDoi().isEmpty()) {
        containerIdentifiers.setDois(dois);
        hasValue = true;
      }
    }

    JsonNode isbnNode = jsonNode.get("isbn");
    if (this.nodeHasTextValue(isbnNode)) {
      containerIdentifiers.setIsbn(isbnNode.textValue());
      hasValue = true;
    }

    JsonNode issnNode = jsonNode.get("issn");
    if (this.nodeHasTextValue(issnNode)) {
      containerIdentifiers.setIssn(issnNode.textValue());
      hasValue = true;
    }

    if (hasValue) {
      return containerIdentifiers;
    } else {
      return null;
    }
  }
}
