package ch.unige.aou.model.xml.deposit.v2_3.deserializer;

import java.io.IOException;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.ObjectCodec;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;

import ch.unige.solidify.exception.SolidifyRuntimeException;

import ch.unige.aou.model.xml.deposit.AouStdDeserializer;
import ch.unige.aou.model.xml.deposit.v2_3.Languages;

/**
 * "links": [
 * {
 * "description": {
 * "lang": "FR",
 * "text": "remplace"
 * },
 * "target": "unige:34567",
 * "type": "INTERNAL"
 * }
 * ],
 */

public class LanguagesDeserializer extends AouStdDeserializer<Languages> {

  private static final long serialVersionUID = 987795L;

  @Override
  public Languages deserialize(JsonParser parser, DeserializationContext context) throws IOException {

    ObjectCodec codec = parser.getCodec();
    JsonNode jsonNode = codec.readTree(parser);

    if (jsonNode instanceof ArrayNode) {
      if (jsonNode.isEmpty()) {
        return null;
      }

      Languages languages = new Languages();
      for (JsonNode languageNode : jsonNode) {
        JsonNode langNode = languageNode.get("language");
        if (this.nodeHasTextValue(langNode)) {
          languages.getLanguage().add(langNode.textValue());
        } else {
          languages.getLanguage()
                  .add(""); // insert empty string b/c the front can send empty field. empty string will help when validating the data.
        }
      }
      return languages;
    } else {
      throw new SolidifyRuntimeException("languages is not an array");
    }
  }
}
