package ch.unige.aou.specification;

import java.util.List;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import ch.unige.solidify.specification.SolidifySpecification;
import ch.unige.solidify.util.StringTool;

import ch.unige.aou.model.settings.License;

public class LicenseSpecification extends SolidifySpecification<License> {
  private static final long serialVersionUID = -1233296744491155033L;

  public LicenseSpecification(License license) {
    super(license);
  }

  @Override
  protected void completePredicatesList(Root<License> root, CriteriaQuery<?> query, CriteriaBuilder builder, List<Predicate> predicatesList) {
    if (this.criteria.getTitle() != null) {
      predicatesList.add(builder.like(root.get("title"), "%" + this.criteria.getTitle() + "%"));
    }
    if (this.criteria.getOpenLicenseId() != null) {
      predicatesList.add(builder.like(root.get("openLicenseId"), "%" + this.criteria.getOpenLicenseId() + "%"));
    }
    if (this.criteria.getVersion() != null) {
      predicatesList.add(builder.like(root.get("version"), "%" + this.criteria.getVersion() + "%"));
    }
    if (this.criteria.getUrl() != null) {
      predicatesList.add(builder.like(root.get("url"), "%" + this.criteria.getUrl() + "%"));
    }
    if (this.criteria.getSortValue() != null) {
      predicatesList.add(builder.equal(root.get("sortValue"), this.criteria.getSortValue()));
    }
    if (this.criteria.getVisible() != null) {
      predicatesList.add(builder.equal(root.get("visible"), this.criteria.getVisible()));
    }

    if (this.criteria.getLicenseGroup() != null && !StringTool.isNullOrEmpty(this.criteria.getLicenseGroup().getResId())) {
      predicatesList.add(builder.equal(root.get("licenseGroup").get("resId"), this.criteria.getLicenseGroup().getResId()));
    }
  }
}
