package ch.unige.aou.specification;

import java.util.List;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Path;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.specification.Join2TiersSpecification;

import ch.unige.aou.model.publication.PublicationStructure;
import ch.unige.aou.model.settings.Structure;

/**
 *
 */
public class PublicationStructureSpecification extends Join2TiersSpecification<PublicationStructure> {

  public PublicationStructureSpecification(PublicationStructure joinCriteria) {
    super(joinCriteria, PublicationStructure.PATH_TO_PUBLICATION, PublicationStructure.PATH_TO_STRUCTURE);
  }

  @Override
  protected String getParentId() {
    return this.joinCriteria.getPublication().getResId();
  }

  @Override
  protected String getChildId() {
    return this.joinCriteria.getStructure().getResId();
  }

  @Override
  protected void completeJoinPredicatesList(Root<PublicationStructure> root, CriteriaQuery<?> query, CriteriaBuilder builder,
          List<Predicate> predicatesList) {
    if (this.joinCriteria.getLinkType() != null) {
      predicatesList.add(builder.equal(root.get("linkType"), this.joinCriteria.getLinkType()));
    }
  }

  @Override
  protected void completeChildPredicatesList(Root<PublicationStructure> root, CriteriaQuery<?> query, CriteriaBuilder builder,
          List<Predicate> predicatesList) {

    Structure structure = this.joinCriteria.getStructure();
    Path structurePath = this.getChildPath(root);

    if (structure.getParentStructure() != null && structure.getParentStructure().getResId() != null) {
      predicatesList.add(builder
              .equal(structurePath.get("parentStructure").get(SolidifyConstants.DB_RES_ID), structure.getParentStructure().getResId()));
    }
    if (structure.getName() != null) {
      predicatesList.add(builder.like(structurePath.get("name"), "%" + structure.getName() + "%"));
    }
    if (structure.getCnStructC() != null) {
      predicatesList.add(builder.equal(structurePath.get("cnStructC"), structure.getCnStructC()));
    }
    if (structure.getCodeStruct() != null) {
      predicatesList.add(builder.equal(structurePath.get("codeStruct"), structure.getCodeStruct()));
    }
    if (structure.getDewey() != null) {
      predicatesList.add(builder.equal(structurePath.get("dewey"), structure.getDewey()));
    }
    if (structure.getStatus() != null) {
      predicatesList.add(builder.equal(structurePath.get("status"), structure.getStatus()));
    }
    if (structure.getSortValue() != null) {
      predicatesList.add(builder.equal(structurePath.get("sortValue"), structure.getSortValue()));
    }
  }
}
