package ch.unige.aou.test.tool;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNull;

import org.junit.jupiter.api.Test;

import ch.unige.aou.model.tool.CleanTool;

class CleanToolTest {

  @Test
  void cleanTitleTest() {
    assertNull(CleanTool.cleanTitle(null));
    assertEquals("", CleanTool.cleanTitle(""));

    assertEquals("A very interesting title", CleanTool.cleanTitle("A very interesting title"));
    assertEquals("A very interesting title", CleanTool.cleanTitle("A very interesting title."));
    assertEquals("A very interesting title", CleanTool.cleanTitle("    A very interesting title   "));
    assertEquals("A very interesting title", CleanTool.cleanTitle("    A    very    interesting    title   "));
    assertEquals("A very interesting title", CleanTool.cleanTitle("\nA very interesting title\n"));
    assertEquals("A very interesting title", CleanTool.cleanTitle("\n\nA very   interesting     title\n\n"));

    assertEquals("A very interesting title", CleanTool.cleanTitle("A very interesting title."));
    assertEquals("A very interesting title", CleanTool.cleanTitle("    A very interesting title.   "));
    assertEquals("A very interesting title", CleanTool.cleanTitle("    A    very    interesting    title.   "));
    assertEquals("A very interesting title", CleanTool.cleanTitle("\nA very interesting title.\n"));
    assertEquals("A very interesting title", CleanTool.cleanTitle("\n\nA very   interesting     title.\n\n"));
    assertEquals("A very interesting title", CleanTool.cleanTitle("\n\nA very \n  interesting  \n   title.\n\n"));

    assertEquals("A very <i>interesting</i> <em>title</em> <sup>sup</sup> <sub>sub</sub>",
            CleanTool.cleanTitle("<s>A</s> <b>very</b> <i>interesting</i> <em>title</em> <sup>sup</sup> <sub>sub</sub>"));
    assertEquals("A very <i>interesting</i> <em>title</em> <sup>sup</sup> <sub>sub</sub>",
            CleanTool.cleanTitle(
                    "<s>A</s> <b>very</b> <i>interesting</i> <script>alert()</script> <em>title</em> <sup>sup</sup> <sub>sub</sub>"));

    assertEquals("A very &gt; interesting title", CleanTool.cleanTitle("A very > interesting title"));
    assertEquals("A very &gt;&gt; interesting title", CleanTool.cleanTitle("A very >> interesting title"));
    assertEquals("A very &gt; interesting title", CleanTool.cleanTitle("A very      > interesting       title       \n"));
    assertEquals("A very &gt; interesting line title", CleanTool.cleanTitle("A very      > interesting <p>line</p>      title       \n"));
    assertEquals("A very &gt; interesting line title", CleanTool.cleanTitle("<p>A very      > interesting <p>line</p>      title       </p>\n"));
    assertEquals("A very &gt; interesting line title",
            CleanTool.cleanTitle("<p><p>A very      > interesting <p>line</p>      title       </p>\n</p>"));
    assertEquals("A very &lt; interesting title", CleanTool.cleanTitle("A very < interesting title"));

    assertEquals("A very interesting title", CleanTool.cleanTitle("A very <test>interesting</test> title"));
    assertEquals("A very interesting title", CleanTool.cleanTitle("A very <test>interesting title"));
    assertEquals("A very &lt;interesting&gt; title", CleanTool.cleanTitle("A very &lt;interesting&gt; title"));
    assertEquals("A very interesting title", CleanTool.cleanTitle("A very <test-script>interesting</test-script> title"));

    assertEquals("A very inte-resting title", CleanTool.cleanTitle("A very <test-script>inte－resting</test-script> title"));
  }

  @Test
  void cleanAbstractTest() {
    assertNull(CleanTool.cleanAbstract(null));
    assertEquals("", CleanTool.cleanAbstract(""));

    assertEquals("A very interesting abstract", CleanTool.cleanAbstract("A very interesting abstract"));
    assertEquals("A very interesting abstract.", CleanTool.cleanAbstract("A very interesting abstract."));
    assertEquals("A very <em>interesting</em> abstract.", CleanTool.cleanAbstract("A very <em>interesting</em> abstract."));

    assertEquals("A very <em>inter-esting</em> abstract.", CleanTool.cleanAbstract("A very <em>inter－esting</em> abstract."));

    assertEquals("A very <em>interesting</em> <a href=\"https://www.unige.ch\">abstract</a>.",
            CleanTool.cleanAbstract("A very <em>interesting</em> <a href=\"https://www.unige.ch\">abstract</a>."));
    assertEquals("A very <em>interesting</em> <a href=\"https://www.unige.ch\" rel=\"noopener noreferrer\" target=\"_blank\">abstract</a>.",
            CleanTool.cleanAbstract(
                    "A very <em>interesting</em> <a href=\"https://www.unige.ch\" rel=\"noopener noreferrer\" target=\"_blank\">abstract</a>."));
    assertEquals("A very <em>interesting</em> <a href=\"https://www.unige.ch\" rel=\"noopener noreferrer\" target=\"_blank\">abstract</a>.",
            CleanTool.cleanAbstract(
                    "A very <em>interesting</em> <a href=\"https://www.unige.ch\" rel=\"noopener noreferrer\" target=\"_blank\" onclick=\"alert();\">abstract</a>."));

    assertEquals("A <sup>very</sup> \n"
                    + "<ul>\n"
                    + " <li>interesting</li>\n"
                    + " <li>clever</li>\n"
                    + "</ul> abstract.",
            CleanTool.cleanAbstract("A <sup>very</sup> <ul><li>interesting</li><li>clever</li></ul> abstract."));

    assertEquals("A <sup>very</sup> \n"
                    + "<ul>\n"
                    + " <li>interesting</li>\n"
                    + " <li>clever</li>\n"
                    + "</ul> abstract.",
            CleanTool.cleanAbstract("      A <sup>very</sup>        <ul><li>interesting</li><li>clever</li></ul>       abstract.        "));

    assertEquals("A <sup>very</sup> \n"
                    + "<ul>\n"
                    + " <li>interesting</li>\n"
                    + " <li>clever</li>\n"
                    + "</ul> abstract. \n"
                    + "<p>with some <sub>new</sub> line</p>",
            CleanTool.cleanAbstract("      A <sup>very</sup>        <ul><li>interesting</li><li>clever</li></ul>       abstract.        \n"
                    + "\n"
                    + "<p>with some <sub>new</sub> line</p>\n"));
  }

  @Test
  void cleanStringTest() {
    assertNull(CleanTool.cleanString(null));
    assertEquals("", CleanTool.cleanString(""));

    assertEquals("A simple string", CleanTool.cleanString("A simple string"));
    assertEquals("A simple string.", CleanTool.cleanString("A simple string."));
    assertEquals("A simple string", CleanTool.cleanString("   A   simple   string"));
    assertEquals("A simple string.", CleanTool.cleanString("A   simple   string.   "));
    assertEquals("A simple string.", CleanTool.cleanString("\n\nA   simple   string.   \n"));
    assertEquals("A simple string.", CleanTool.cleanString("\n\nA   simple \n\n  string.   \n"));

    assertEquals("<sup>A</sup> s<em>imp</em>l<b>e<b> <p>string</p>.",
            CleanTool.cleanString("\n\n<sup>A</sup>   s<em>imp</em>l<b>e<b> \n\n  <p>string</p>.   \n"));
    assertEquals("A very > interesting <p>line</p> title", CleanTool.cleanString("A very      > interesting <p>line</p>      title       \n"));

  }

  @Test
  void capitalizeStringTest() {
    assertEquals("François-Xavier D'Abzac", CleanTool.capitalizeString("FRANÇOIS-XAVIER D'ABZAC"));
    assertEquals("A D", CleanTool.capitalizeString("A D"));
    assertEquals("A d", CleanTool.capitalizeString("A d"));
    assertEquals("Joao da Silva", CleanTool.capitalizeString("Joao da Silva"));
    assertEquals("Marie Dupont-Pictet", CleanTool.capitalizeString("MARIE DUPONT-PICTET"));
    assertEquals("Marie Dupont-Pict'E", CleanTool.capitalizeString("MARIE DUPONT-PICT'E"));
  }

  @Test
  void cleanPagingTest() {
    assertNull(CleanTool.cleanPaging(null));
    assertEquals("", CleanTool.cleanPaging(""));

    assertEquals("143-145", CleanTool.cleanPaging("143-145"));
    assertEquals("143-145", CleanTool.cleanPaging("      143-145"));
    assertEquals("143-145", CleanTool.cleanPaging("   143-145  "));
    assertEquals("143-145", CleanTool.cleanPaging("   143  -  145  "));
    assertEquals("143-145", CleanTool.cleanPaging("   143  -  45  "));
    assertEquals("143-145", CleanTool.cleanPaging("   143  -\t45  "));
    assertEquals("143-145", CleanTool.cleanPaging("   143 \t -\t 45  "));

    assertEquals("143-143", CleanTool.cleanPaging("143-43"));
    assertEquals("143-143", CleanTool.cleanPaging(" 143  - 43 "));
    assertEquals("143-143", CleanTool.cleanPaging(" 143  -\t 43 "));
    assertEquals("143-143", CleanTool.cleanPaging(" \t143 \t - 43 "));

    assertEquals("p. 143-145", CleanTool.cleanPaging("p. 143-145"));
    assertEquals("p. 143-145", CleanTool.cleanPaging("p.      143-145"));
    assertEquals("p. 143-145", CleanTool.cleanPaging("   p.   143-145"));
    assertEquals("p. 143-145", CleanTool.cleanPaging("   p. \t  143-145\t"));

    assertEquals("p. 143-145", CleanTool.cleanPaging("p. 143-45"));
    assertEquals("p. 143-145", CleanTool.cleanPaging("p. 143-   45"));
    assertEquals("p. 143-145", CleanTool.cleanPaging("p. 143   -45"));
    assertEquals("p. 143-145", CleanTool.cleanPaging("p. 143\t   -45\t"));

    assertEquals("p. 143-145", CleanTool.cleanPaging("p. 143-5"));
    assertEquals("p. 143-145", CleanTool.cleanPaging("p. 143-   5"));
    assertEquals("p. 143-145", CleanTool.cleanPaging("p. 143   -5"));
    assertEquals("p. 143-145", CleanTool.cleanPaging("p. 143\t\t-5\t\t\t"));

    assertEquals("p. 143-145 test", CleanTool.cleanPaging("p. 143-   5   test"));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("p. 143-   45   test"));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("p. 143-   145   test"));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("   p. 143-   145   test  "));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging(" \t  p.\t143-\t\t\t145\t\t\ttest\t"));

    assertEquals("p. 143-145 test", CleanTool.cleanPaging("p. 143   -5   test"));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("p. 143   -45   test"));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("p. 143   -145   test"));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("   p. 143   -145   test   "));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("   p.\t 143   -145   test   \t"));

    assertEquals("p. 143-145 test", CleanTool.cleanPaging("p. 143   -  5   test"));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("p. 143   -  45   test"));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("p. 143   -  145   test"));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("   p. 143   -  145   test  "));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("   p. 143\t   - \t 145   test  "));

    // other kinds of hyphens
    assertEquals("143-145", CleanTool.cleanPaging("   143  ‒  45  "));
    assertEquals("143-145", CleanTool.cleanPaging("   143  ‑  45  "));
    assertEquals("143-145", CleanTool.cleanPaging("   143  －  45  "));
    assertEquals("143-145", CleanTool.cleanPaging("  \t 143  － \t 45 \t "));
    assertEquals("143-145", CleanTool.cleanPaging("   143  ‒  45  "));

    assertEquals("123456785-123456789", CleanTool.cleanPaging("   123456785  ‒  9  "));
  }

  @Test
  void doNotCleanPagingTest() {
    assertEquals("143-27", CleanTool.cleanPaging("143-27"));
    assertEquals("1243-145", CleanTool.cleanPaging("1243-145"));
    assertEquals("1143-test", CleanTool.cleanPaging("1143-test"));
    assertEquals("bla 1-1 test", CleanTool.cleanPaging("bla 1-1 test"));
    assertEquals("lorem ipsum dolores", CleanTool.cleanPaging("lorem ipsum dolores"));
  }

  @Test
  void cleanStringAndHyphens() {
    assertNull(CleanTool.cleanStringAndHyphens(null));
    assertEquals("", CleanTool.cleanStringAndHyphens(""));

    String expectedResult = "xxx-yy-z";
    assertEquals(expectedResult, CleanTool.cleanStringAndHyphens(expectedResult));
    assertEquals(expectedResult, CleanTool.cleanStringAndHyphens("xxx - yy - z"));
    assertEquals(expectedResult, CleanTool.cleanStringAndHyphens("    xxx - yy - z  "));
    assertEquals(expectedResult, CleanTool.cleanStringAndHyphens("    xxx ‒ yy ‒ z  "));
    assertEquals(expectedResult, CleanTool.cleanStringAndHyphens("xxx － yy － z"));
    assertEquals(expectedResult, CleanTool.cleanStringAndHyphens("xxx －    yy –    z   "));
  }

  @Test
  void cleanTrailingDots() {
    assertNull(CleanTool.removeTrailingDots(null));
    assertEquals("", CleanTool.removeTrailingDots(""));
    assertEquals("A", CleanTool.removeTrailingDots("A."));
    assertEquals("A", CleanTool.removeTrailingDots("A.."));
    assertEquals("A.B", CleanTool.removeTrailingDots("A.B"));
    assertEquals("A.B", CleanTool.removeTrailingDots("A.B."));
    assertEquals("A..B", CleanTool.removeTrailingDots("A..B."));
  }

  @Test
  void removeTrailingColons() {
    assertNull(CleanTool.removeTrailingColon(null));
    assertEquals("A simple string", CleanTool.removeTrailingColon("A simple string"));
    assertEquals("A simple string", CleanTool.removeTrailingColon("A simple string:"));
    assertEquals("A simple string", CleanTool.removeTrailingColon("A simple string:::"));
    assertEquals("A simple: string", CleanTool.removeTrailingColon("A simple: string:::"));
    assertEquals(":A simple: string", CleanTool.removeTrailingColon(":A simple: string:::"));
  }

  @Test
  void cleanTrailingParenthesis() {
    assertNull(CleanTool.removeTrailingParenthesisAndContent(null));
    assertEquals("", CleanTool.removeTrailingParenthesisAndContent(""));
    assertEquals("A", CleanTool.removeTrailingParenthesisAndContent("A()"));
    assertEquals("A", CleanTool.removeTrailingParenthesisAndContent("A(AVC)"));
    assertEquals("A(AVC)B", CleanTool.removeTrailingParenthesisAndContent("A(AVC)B"));
  }

  @Test
  void cleanInvisibleCharacters() {
    assertNull(CleanTool.removeInvisibleCharacters(null));
    assertEquals("", CleanTool.removeInvisibleCharacters(""));
    assertEquals("Die Volkswirtschaft", CleanTool.removeInvisibleCharacters("\u0098Die \u009CVolkswirtschaft\n"));
    assertEquals("I Quaderni dell'ingegnere", CleanTool.removeInvisibleCharacters("\u0098I \u009CQuaderni dell'ingegnere\n"));
  }
}
