package ch.unige.aou.test.tool;

import static org.junit.jupiter.api.Assertions.assertNotNull;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.namespace.QName;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import ch.unige.aou.model.xml.fedora.foxml.v1.DigitalObject;
import ch.unige.aou.model.xml.fedora.repository.DatastreamProfile;
import ch.unige.aou.model.xml.fedora.repository.FedoraRepository;
import ch.unige.aou.model.xml.oai.v2.IdentifyType;
import ch.unige.aou.model.xml.oai.v2.OAIPMHtype;

@ExtendWith(SpringExtension.class)
class JAXBTest extends AbstractXMLTest {
  private static final Logger log = LoggerFactory.getLogger(JAXBTest.class);
  private static final String FOXML_1_1_FILE = "foxml_reference_example.xml";
  private static final String FEDORA3_DS_PROFILE_FILE = "dataStreamProfile.xml";
  private static final String FEDORA3_REPOSITORY_FILE = "fedoraRepository.xml";

  @Test
  void createOAIPMHTest() throws Exception {
    final JAXBContext jaxbContext = JAXBContext.newInstance(OAIPMHtype.class);
    final Marshaller jaxbMarshaller = jaxbContext.createMarshaller();
    jaxbMarshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);

    log.info("Java => OAI-PMH tests");
    final OAIPMHtype oai = new OAIPMHtype();
    final IdentifyType id = new IdentifyType();
    id.setBaseURL("https://archive-ouverte.unige.ch/");
    id.setRepositoryName("AoU");
    oai.setIdentify(id);
    final QName qName = new QName("http://www.openarchives.org/OAI/2.0/", "OAI-PMH");
    final JAXBElement<OAIPMHtype> root = new JAXBElement<>(qName, OAIPMHtype.class, oai);
    assertNotNull(root);
    jaxbMarshaller.marshal(root, System.out);
  }

  @Test
  void readRepositoryInfoTest() throws JAXBException, FileNotFoundException, IOException {
    this.checkXmlSerialization(FEDORA3_REPOSITORY_FILE, FedoraRepository.class, true);
  }

  @Test
  void readFoxmlTest() throws FileNotFoundException, JAXBException, IOException {
    this.checkXmlSerialization(FOXML_1_1_FILE, DigitalObject.class, true);
  }

  @Test
  void readDatastreamTest() throws Exception {
    this.checkXmlSerialization(FEDORA3_DS_PROFILE_FILE, DatastreamProfile.class, false);
  }

  @SuppressWarnings("unchecked")
  private <T> void checkXmlSerialization(String xml, Class<T> objType, boolean print) throws JAXBException, FileNotFoundException, IOException {
    final JAXBContext jaxbContext = JAXBContext.newInstance(objType);
    final Marshaller jaxbMarshaller = jaxbContext.createMarshaller();
    final Unmarshaller jaxbUnmarshaller = jaxbContext.createUnmarshaller();
    T obj;
    try (FileReader fileReader = new FileReader(this.getPath(xml).toFile())) {
      obj = (T) jaxbUnmarshaller.unmarshal(fileReader);
      assertNotNull(obj);
      if (print) {
        jaxbMarshaller.marshal(obj, System.out);
      }
    }
  }
}
