package ch.unige.aou.business;

import org.springframework.context.annotation.Profile;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Service;

import ch.unige.solidify.exception.SolidifyResourceNotFoundException;
import ch.unige.solidify.exception.SolidifyRuntimeException;

import ch.unige.aou.model.security.User;
import ch.unige.aou.service.rest.UserRemoteResourceService;

@Profile("!test")
@Service
public class AuthenticatedUserSecurity implements AuthenticatedUserProvider {

  private final UserRemoteResourceService userResourceService;

  public AuthenticatedUserSecurity(UserRemoteResourceService userResourceService) {
    this.userResourceService = userResourceService;
  }

  @Override
  public String getPersonId(Authentication authentication) {
    final String authUserId = authentication.getName();

    /*
     * Get Person linked to user identified by its OAuth2 token
     */
    final User user = this.userResourceService.getAuthenticatedUser();
    if (user.getExternalUid().equals(authUserId)) {
      if (user.getPerson() != null) {
        return user.getPerson().getResId();
      } else {
        throw new SolidifyResourceNotFoundException("Authenticated user is not linked to any person");
      }
    } else {
      throw new SolidifyRuntimeException("User returned does not correspond to authentication");
    }
  }
}
