package ch.unige.aou.business;

import java.util.List;
import java.util.Optional;

import org.springframework.stereotype.Service;

import ch.unige.solidify.specification.SolidifySpecification;
import ch.unige.solidify.util.StringTool;

import ch.unige.aou.model.publication.Contributor;
import ch.unige.aou.model.publication.ContributorDTO;
import ch.unige.aou.repository.ContributorRepository;
import ch.unige.aou.specification.ContributorSpecification;

@Service
public class ContributorService extends AouResourceService<Contributor> {

  private final ContributorRepository contributorRepository;

  public ContributorService(ContributorRepository contributorRepository) {
    this.contributorRepository = contributorRepository;
  }

  @Override
  public SolidifySpecification<Contributor> getSpecification(Contributor resource) {
    return new ContributorSpecification(resource);
  }

  public Contributor findOrCreate(ContributorDTO contributorDTO) {
    Contributor existingContributor = null;

    if (StringTool.isNullOrEmpty(contributorDTO.getCnIndividu())) {
      existingContributor = this.contributorRepository
              .findByFirstNameAndLastNameAndCnIndividuIsNull(contributorDTO.getFirstName(), contributorDTO.getLastName());
    } else {
      existingContributor = this.contributorRepository.findByCnIndividu(contributorDTO.getCnIndividu());
    }

    if (existingContributor != null) {
      return existingContributor;
    } else {
      Contributor newContributor = new Contributor();
      newContributor.setFirstName(contributorDTO.getFirstName());
      newContributor.setLastName(contributorDTO.getLastName());
      newContributor.setCnIndividu(contributorDTO.getCnIndividu());
      return this.save(newContributor);
    }
  }

  public Optional<Contributor> findByResIdAndPublicationsIsNull(String contributorId) {
    return this.contributorRepository.findByResIdAndPublicationsIsNull(contributorId);
  }

  /**
   * Delete eventual contributors that are not linked to any Publication anymore.
   *
   * @param contributorIds The contributors to check
   */
  public void deleteContributorsIfOrphan(List<String> contributorIds) {
    for (String previousContributorId : contributorIds) {
      Optional<Contributor> orphanContributor = this.findByResIdAndPublicationsIsNull(previousContributorId);
      if (orphanContributor.isPresent()) {
        this.delete(orphanContributor.get().getResId());
      }
    }
  }
}
