package ch.unige.aou.business;

import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import ch.unige.aou.AouConstants;
import ch.unige.aou.model.Sortable;
import ch.unige.aou.model.security.Role;
import ch.unige.aou.model.settings.Label;
import ch.unige.aou.repository.RoleRepository;
import ch.unige.aou.specification.RoleSpecification;

@Service
public class RoleService extends AouResourceService<Role> {
  private static final Logger log = LoggerFactory.getLogger(RoleService.class);

  private RoleRepository roleRepository;
  private LabeledLanguageService labeledLanguageService;

  public RoleService(RoleRepository roleRepository, LabeledLanguageService labeledLanguageService) {
    this.roleRepository = roleRepository;
    this.labeledLanguageService = labeledLanguageService;
  }

  @Override
  public RoleSpecification getSpecification(Role resource) {
    return new RoleSpecification(resource);
  }

  @Override
  public Optional<Integer> getMaxSortValue(Sortable item) {
    return ((RoleRepository) this.itemRepository).findMaxSortValue();
  }

  @Override
  protected void beforeValidate(Role item) {
    this.setDefaultSortValue(item);
  }

  public void initDefaultData() {
    if (!this.roleRepository.findById(AouConstants.ROLE_SUPER_VALIDATOR_ID).isPresent()) {
      Role role = new Role();
      role.setResId(AouConstants.ROLE_SUPER_VALIDATOR_ID);
      role.setName("Super Validator");
      role.getLabels().add(new Label("Super validator", this.labeledLanguageService.findByCode(AouConstants.LANG_CODE_ENGLISH)));
      role.getLabels().add(new Label("Super validateur", this.labeledLanguageService.findByCode(AouConstants.LANG_CODE_FRENCH)));
      this.save(role);
      log.info("Role '{}' created", role.getName());
    }
  }
}
