package ch.unige.aou.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.event.EventListener;
import org.springframework.jms.core.JmsTemplate;

import ch.unige.solidify.service.MessageService;

import ch.unige.aou.config.AouProperties;
import ch.unige.aou.message.EmailMessage;
import ch.unige.aou.message.PublicationIndexingMessage;
import ch.unige.aou.message.PublicationMessage;
import ch.unige.aou.message.PublicationMetadataUpgradeMessage;

public abstract class JmsService extends AouService {
  private static final Logger log = LoggerFactory.getLogger(JmsService.class);

  @Autowired
  private JmsTemplate queueJmsTemplate;

  @Autowired
  @Qualifier("topicJmsTemplate")
  private JmsTemplate topicJmsTemplate;

  private final String publicationsQueue;
  private final String publicationsIndexingQueue;
  private final String publicationsMetadataUpgradeQueue;
  private final String emailsQueue;

  public JmsService(AouProperties aouProperties, MessageService messageService) {
    super(messageService);

    this.publicationsQueue = aouProperties.getQueue().getPublications();
    this.publicationsIndexingQueue = aouProperties.getQueue().getPublicationsIndexing();
    this.publicationsMetadataUpgradeQueue = aouProperties.getQueue().getPublicationsMetadataUpgrade();
    this.emailsQueue = aouProperties.getQueue().getEmails();
  }

  protected void convertAndSendToQueue(String queue, Object event) {
    if (log.isTraceEnabled()) {
      log.trace("Sending message to queue {} => {}", queue, event);
    }
    this.queueJmsTemplate.convertAndSend(queue, event);
  }

  protected void convertAndSendToTopic(String topic, Object event) {
    if (log.isTraceEnabled()) {
      log.trace("Sending message to topic {} => {}", topic, event);
    }
    this.topicJmsTemplate.convertAndSend(topic, event);
  }

  @EventListener
  private void sendMessage(PublicationMessage publicationMessage) {
    this.convertAndSendToQueue(this.publicationsQueue, publicationMessage);
  }

  @EventListener
  private void sendMessage(PublicationIndexingMessage publicationIndexingMessage) {
    this.convertAndSendToQueue(this.publicationsIndexingQueue, publicationIndexingMessage);
  }

  @EventListener
  private void sendMessage(PublicationMetadataUpgradeMessage publicationMetadataUpgradeMessage) {
    this.convertAndSendToQueue(this.publicationsMetadataUpgradeQueue, publicationMetadataUpgradeMessage);
  }

  @EventListener
  private void sendMessage(EmailMessage emailMessage) {
    this.convertAndSendToQueue(this.emailsQueue, emailMessage);
  }
}
