package ch.unige.aou.service.metadata;

import java.io.Serializable;
import java.io.StringReader;
import java.io.StringWriter;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import javax.xml.bind.JAXB;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.Marshaller;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;

import org.springframework.stereotype.Service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectReader;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.databind.module.SimpleModule;

import ch.unige.solidify.exception.SolidifyResourceNotFoundException;
import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.exception.SolidifyValidationException;
import ch.unige.solidify.service.MessageService;
import ch.unige.solidify.util.StringTool;
import ch.unige.solidify.validation.ValidationError;

import ch.unige.aou.business.ContributorService;
import ch.unige.aou.business.LabeledLanguageService;
import ch.unige.aou.business.PersonService;
import ch.unige.aou.business.PublicationSubSubtypeService;
import ch.unige.aou.business.PublicationSubtypeService;
import ch.unige.aou.business.PublicationTypeService;
import ch.unige.aou.business.ResearchGroupService;
import ch.unige.aou.business.StructureService;
import ch.unige.aou.config.AouProperties;
import ch.unige.aou.model.publication.AbstractContributor;
import ch.unige.aou.model.publication.CollaborationDTO;
import ch.unige.aou.model.publication.ContributorDTO;
import ch.unige.aou.model.publication.DocumentFile;
import ch.unige.aou.model.publication.MetadataCollection;
import ch.unige.aou.model.publication.MetadataCorrection;
import ch.unige.aou.model.publication.MetadataDates;
import ch.unige.aou.model.publication.MetadataFile;
import ch.unige.aou.model.publication.MetadataFunding;
import ch.unige.aou.model.publication.MetadataLink;
import ch.unige.aou.model.publication.MetadataTextLang;
import ch.unige.aou.model.publication.Publication;
import ch.unige.aou.model.settings.LabeledLanguage;
import ch.unige.aou.model.settings.ResearchGroup;
import ch.unige.aou.model.settings.Structure;
import ch.unige.aou.model.xml.deposit.v1.Abstracts;
import ch.unige.aou.model.xml.deposit.v1.AcademicStructure;
import ch.unige.aou.model.xml.deposit.v1.AcademicStructures;
import ch.unige.aou.model.xml.deposit.v1.AccessLevel;
import ch.unige.aou.model.xml.deposit.v1.AuthorRole;
import ch.unige.aou.model.xml.deposit.v1.Citations;
import ch.unige.aou.model.xml.deposit.v1.Classifications;
import ch.unige.aou.model.xml.deposit.v1.Collaboration;
import ch.unige.aou.model.xml.deposit.v1.Collections;
import ch.unige.aou.model.xml.deposit.v1.Container;
import ch.unige.aou.model.xml.deposit.v1.ContainerIdentifiers;
import ch.unige.aou.model.xml.deposit.v1.Contributor;
import ch.unige.aou.model.xml.deposit.v1.Contributors;
import ch.unige.aou.model.xml.deposit.v1.Dataset;
import ch.unige.aou.model.xml.deposit.v1.DateWithType;
import ch.unige.aou.model.xml.deposit.v1.Dates;
import ch.unige.aou.model.xml.deposit.v1.DepositDoc;
import ch.unige.aou.model.xml.deposit.v1.DepositIdentifiers;
import ch.unige.aou.model.xml.deposit.v1.Dois;
import ch.unige.aou.model.xml.deposit.v1.Embargo;
import ch.unige.aou.model.xml.deposit.v1.File;
import ch.unige.aou.model.xml.deposit.v1.FileType;
import ch.unige.aou.model.xml.deposit.v1.Files;
import ch.unige.aou.model.xml.deposit.v1.Funding;
import ch.unige.aou.model.xml.deposit.v1.Fundings;
import ch.unige.aou.model.xml.deposit.v1.Group;
import ch.unige.aou.model.xml.deposit.v1.Groups;
import ch.unige.aou.model.xml.deposit.v1.Keywords;
import ch.unige.aou.model.xml.deposit.v1.Link;
import ch.unige.aou.model.xml.deposit.v1.LinkTypes;
import ch.unige.aou.model.xml.deposit.v1.Links;
import ch.unige.aou.model.xml.deposit.v1.ObjectFactory;
import ch.unige.aou.model.xml.deposit.v1.Pages;
import ch.unige.aou.model.xml.deposit.v1.Publisher;
import ch.unige.aou.model.xml.deposit.v1.Rights;
import ch.unige.aou.model.xml.deposit.v1.Text;
import ch.unige.aou.model.xml.deposit.v1.TextLang;
import ch.unige.aou.model.xml.deposit.v1.deserializer.AbstractsDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.AcademicsStructuresDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.CitationsDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.ClassificationsDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.CollectionsDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.ContainerDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.ContainerIdentifiersDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.ContributorsDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.DatasetDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.DateWithTypeDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.DatesDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.DepositIdentifiersDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.DoisDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.FilesDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.FundingsDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.GroupsDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.KeywordsDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.LinksDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.PagesDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.PublisherDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.RightsDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.TextDeserializer;
import ch.unige.aou.model.xml.deposit.v1.deserializer.TextLangDeserializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.AbstractsSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.AcademicStructuresSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.AuthorRoleSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.CitationsSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.ClassificationsSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.CollectionsSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.ContainerIdentifiersSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.ContributorsSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.DatasetSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.DateWithTypeSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.DatesSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.DepositDocSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.DepositIdentifiersSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.DoisSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.FilesSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.FundingsSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.GroupsSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.KeywordsSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.LinksSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.PagesSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.PublisherSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.RightsSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.TextLangSerializer;
import ch.unige.aou.model.xml.deposit.v1.serializer.TextSerializer;
import ch.unige.aou.service.rest.JournalTitleRemoteService;

@Service
public class MetadataExtractorV1 extends MetadataExtractor {

  ObjectFactory factory = new ObjectFactory();

  public MetadataExtractorV1(AouProperties aouProperties, PublicationTypeService publicationTypeService,
          PublicationSubtypeService publicationSubtypeService, PublicationSubSubtypeService publicationSubSubtypeService,
          StructureService structureService, ResearchGroupService researchGroupService, ContributorService contributorService,
          LabeledLanguageService labeledLanguageService, JournalTitleRemoteService journalTitleRemoteService, PersonService personService,
          MessageService messageService) {
    super(aouProperties, publicationTypeService, publicationSubtypeService, publicationSubSubtypeService, structureService, researchGroupService,
            contributorService, labeledLanguageService, journalTitleRemoteService, personService, messageService);
  }

  @Override
  public String transformFormDataToMetadata(String jsonData) {
    DepositDoc depositDoc = this.createDepositDocFromJsonFormData(jsonData);
    return this.serializeDepositDocToXml(depositDoc);
  }

  @Override
  public String getPmcIdFromPublication(Publication publication) {
    DepositDoc depositDoc = (DepositDoc) this.createDepositDocObjectFromXml(publication.getMetadata());
    return this.getPmcId(depositDoc);
  }

  @Override
  public DepositDoc createDepositDocFromJsonFormData(String jsonData) {

    try {
      ObjectMapper mapper = this.getJsonFormDeserializerMapper();

      JsonNode root = mapper.readTree(jsonData);
      JsonNode typeNode = root.at("/type");
      JsonNode contributorsNode = root.at("/contributors");
      JsonNode descriptionNode = root.at("/description");

      DepositDoc depositDoc = mapper.readValue(typeNode.toString(), DepositDoc.class);
      ObjectReader updater = mapper.readerForUpdating(depositDoc);
      depositDoc = updater.readValue(contributorsNode.toString());
      updater = mapper.readerForUpdating(depositDoc);
      depositDoc = updater.readValue(descriptionNode.toString());

      this.replaceFormValuesByMetadataValues(depositDoc);

      return depositDoc;

    } catch (JsonProcessingException e) {
      if (e.getCause() instanceof SolidifyValidationException) {
        throw (SolidifyValidationException) e.getCause();
      }
      throw new SolidifyRuntimeException("unable to parse json data", e);
    }
  }

  @Override
  public String transformMetadataToFormData(String xmlData) {
    DepositDoc depositDoc = (DepositDoc) this.createDepositDocObjectFromXml(xmlData);

    this.replaceMetadataValuesByFormValues(depositDoc);

    try {
      ObjectMapper mapper = this.getJsonFormSerializerMapper();
      return mapper.writeValueAsString(depositDoc);
    } catch (JsonProcessingException e) {
      throw new SolidifyRuntimeException("unable to transform XML data to Json form data", e);
    }
  }

  /**
   * Save the document files properties into the metadata content
   */
  @Override
  public void completeMetadataWithFiles(Publication publication) {
    DepositDoc depositDoc = (DepositDoc) this.createDepositDocObjectFromXml(publication.getMetadata());

    Files files = new Files();
    for (DocumentFile df : publication.getDocumentFiles()) {
      File xmlFile = this.convertDocumentFileToXmlFile(df);
      files.getFile().add(xmlFile);
    }
    depositDoc.setFiles(files);
    //add into the publication metadata the info from file
    String xmlMetadata = this.serializeDepositDocToXml(depositDoc);
    publication.setMetadata(xmlMetadata);
  }

  @Override
  public void completeClassificationsMetadataWithDeweyCodes(Publication publication) {
    // do nothing in V1
  }

  @Override
  protected void formatDepositDocDatesToFrontendFormat(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getDates() != null && depositDoc.getDates().getDate() != null) {
      List<DateWithType> dates = depositDoc.getDates().getDate();
      for (DateWithType date : dates) {
        if (!StringTool.isNullOrEmpty(date.getContent())) {
          String dateStr = this.updateDateFromMetadataToFrontendFormat(date.getContent());
          date.setContent(dateStr);
        }
      }
    }
  }

  @Override
  protected void formatDepositDocDatesToMetadataFormat(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getDates() != null && depositDoc.getDates().getDate() != null) {
      List<DateWithType> dates = depositDoc.getDates().getDate();
      for (DateWithType date : dates) {
        if (!StringTool.isNullOrEmpty(date.getContent())) {
          String dateStr = this.updateDateFromFrontendToMetadataFormat(date.getContent());
          date.setContent(dateStr);
        }
      }
    }
  }

  @Override
  public String serializeDepositDocToXml(Object depositDocObject) {
    try {
      DepositDoc depositDoc = (DepositDoc) depositDocObject;
      JAXBContext jaxbContext = JAXBContext.newInstance(DepositDoc.class);
      Marshaller jaxbMarshaller = jaxbContext.createMarshaller();
      jaxbMarshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
      StringWriter sw = new StringWriter();
      jaxbMarshaller.marshal(depositDoc, sw);
      return sw.toString();

    } catch (Exception e) {
      throw new SolidifyRuntimeException("unable to serialize DepositDoc to XML", e);
    }
  }

  @Override
  public Object createDepositDocObjectFromXml(String xmlData) {
    if (xmlData == null) {
      throw new SolidifyValidationException(new ValidationError("XmlData is null. Unable to deserialize XML to DepositDoc."));
    }
    return JAXB.unmarshal(new StringReader(xmlData), DepositDoc.class);
  }

  @Override
  public void replaceIdsByTextValues(Object depositDocObj) {
    super.replaceIdsByTextValues(depositDocObj);

    /*
     * Language
     */
    String languageStr = this.getLanguage(depositDocObj);
    if (!StringTool.isNullOrEmpty(languageStr)) {
      Optional<LabeledLanguage> language = this.labeledLanguageService.findById(languageStr);
      language.ifPresent(lang -> this.setLanguage(depositDocObj, lang.getCode()));
    }

    /*
     * Container's title language
     */
    String containerTitleLangStr = this.getContainerTitleLang(depositDocObj);
    if (!StringTool.isNullOrEmpty(containerTitleLangStr)) {
      Optional<LabeledLanguage> language = this.labeledLanguageService.findById(containerTitleLangStr);
      language.ifPresent(lang -> this.setContainerTitleLang(depositDocObj, lang.getCode()));
    }

    /*
     * Citations' languages
     */
    List<MetadataTextLang> citations = this.getCitations(depositDocObj);
    if (citations != null) {
      for (MetadataTextLang citation : citations) {
        if (!StringTool.isNullOrEmpty(citation.getLang())) {
          Optional<LabeledLanguage> language = this.labeledLanguageService.findById(citation.getLang());
          language.ifPresent(lang -> citation.setLang(lang.getCode()));
        }
      }
      this.setCitations(depositDocObj, citations);
    }
  }

  @Override
  public void replaceTextValuesByIds(Object depositDocObj) {
    super.replaceTextValuesByIds(depositDocObj);

    /*
     * Language
     */
    String languageStr = this.getLanguage(depositDocObj);
    if (!StringTool.isNullOrEmpty(languageStr)) {
      LabeledLanguage language = this.labeledLanguageService.findByCode(languageStr);
      if (language != null) {
        this.setLanguage(depositDocObj, language.getResId());
      }
    }

    /*
     * Container's title language
     */
    String containerTitleLangStr = this.getContainerTitleLang(depositDocObj);
    if (!StringTool.isNullOrEmpty(containerTitleLangStr)) {
      LabeledLanguage language = this.labeledLanguageService.findByCode(containerTitleLangStr);
      if (language != null) {
        this.setContainerTitleLang(depositDocObj, language.getResId());
      }
    }

    /*
     * Citations' languages
     */
    List<MetadataTextLang> citations = this.getCitations(depositDocObj);
    if (citations != null) {
      for (MetadataTextLang citation : citations) {
        if (!StringTool.isNullOrEmpty(citation.getLang())) {
          LabeledLanguage language = this.labeledLanguageService.findByCode(citation.getLang());
          if (language != null) {
            citation.setLang(language.getResId());
          }
        }
      }
      this.setCitations(depositDocObj, citations);
    }
  }

  @Override
  public void addResearchGroup(ResearchGroup researchGroup, Publication publication) {
    DepositDoc depositDoc = this.createDepositDocFromJsonFormData(publication.getFormData());
    Group group = new Group();
    group.setName(researchGroup.getName());
    group.setCode(researchGroup.getCode() != null ? new BigInteger(researchGroup.getCode()) : null);

    Groups depositDocGroups = depositDoc.getGroups();
    if (!depositDocGroups.getGroup().contains(group)) {
      depositDocGroups.getGroup().add(group);
    }

    String metadata = this.serializeDepositDocToXml(depositDoc);
    publication.setMetadata(metadata);
  }

  @Override
  public void addStructure(Structure structure, Publication publication) {
    DepositDoc depositDoc = (DepositDoc) this.createDepositDocObjectFromXml(publication.getMetadata());

    // create a new AcademicStructure object
    AcademicStructure academicStructure = new AcademicStructure();
    academicStructure.setCode(structure.getCodeStruct());
    academicStructure.setName(structure.getName());

    if (depositDoc.getAcademicStructures() != null) {
      final boolean existingStructure = depositDoc.getAcademicStructures().getAcademicStructure().stream()
              .anyMatch(acst -> acst.getCode().equals(structure.getCodeStruct()));

      if (!existingStructure) {
        depositDoc.getAcademicStructures().getAcademicStructure().add(academicStructure);
      }
    } else {
      // create a AcademicStructures
      AcademicStructures academicStructures = new AcademicStructures();
      depositDoc.setAcademicStructures(academicStructures);
      depositDoc.getAcademicStructures().getAcademicStructure().add(academicStructure);
    }

    String metadata = this.serializeDepositDocToXml(depositDoc);
    publication.setMetadata(metadata);
  }

  @Override
  public void addLanguage(LabeledLanguage labeledLanguage, Publication publication) {
    DepositDoc depositDoc = (DepositDoc) this.createDepositDocObjectFromXml(publication.getMetadata());
    depositDoc.setLanguage(labeledLanguage.getCode());
    String metadata = this.serializeDepositDocToXml(depositDoc);
    publication.setMetadata(metadata);
  }

  @Override
  public boolean isCnIndividuInContributors(Publication publication, String cnIndividu) {
    if (StringTool.isNullOrEmpty(cnIndividu)) {
      return false;
    }

    DepositDoc depositDoc = (DepositDoc) this.createDepositDocObjectFromXml(publication.getMetadata());
    List<AbstractContributor> contributors = this.getContributors(depositDoc);
    return contributors.stream().anyMatch(abstractContributor -> abstractContributor instanceof ContributorDTO
            && cnIndividu.equals(((ContributorDTO) abstractContributor).getCnIndividu()));
  }

  @Override
  public void removeCnIndividuFromContributors(Publication publication, String cnIndividu) {
    DepositDoc depositDoc = (DepositDoc) this.createDepositDocObjectFromXml(publication.getMetadata());

    // loop on all contributors and replace cnIndividu by null if it matches
    List<AbstractContributor> contributors = this.getContributors(depositDoc);
    for (AbstractContributor abstractContributor : contributors) {
      if (abstractContributor instanceof ContributorDTO) {
        ContributorDTO contributorDTO = (ContributorDTO) abstractContributor;
        if (contributorDTO.getCnIndividu() != null && contributorDTO.getCnIndividu().equals(cnIndividu)) {
          contributorDTO.setCnIndividu(null);
        }
      }
    }
    this.setContributors(depositDoc, contributors);
    String metadata = this.serializeDepositDocToXml(depositDoc);
    publication.setMetadata(metadata);
  }

  @Override
  public void cleanMetadata(Publication publication) {
    // does nothing in V1
  }

  @Override
  public void cleanMetadataToClone(Publication publication) {
    // does nothing in V1
  }

  /*******************************************************************************************/

  @Override
  public String getType(Object depositDocObj) {
    return ((DepositDoc) depositDocObj).getType();
  }

  @Override
  public void setType(Object depositDocObj, String value) {
    ((DepositDoc) depositDocObj).setType(value);
  }

  @Override
  public String getSubtype(Object depositDocObj) {
    return ((DepositDoc) depositDocObj).getSubtype();
  }

  @Override
  void setSubtype(Object depositDocObj, String value) {
    ((DepositDoc) depositDocObj).setSubtype(value);
  }

  @Override
  public String getSubSubtype(Object depositDocObj) {
    return ((DepositDoc) depositDocObj).getSubsubtype();
  }

  @Override
  void setSubSubtype(Object depositDocObj, String value) {
    ((DepositDoc) depositDocObj).setSubsubtype(value);
  }

  @Override
  public String getTitleContent(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getTitle() != null) {
      return depositDoc.getTitle().getContent();
    } else {
      return null;
    }
  }

  public String getLanguage(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getLanguage() != null) {
      return depositDoc.getLanguage();
    } else {
      return null;
    }
  }

  void setLanguage(Object depositDocObj, String value) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    depositDoc.setLanguage(value);
  }

  @Override
  public String getTitleLang(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getTitle() != null) {
      return depositDoc.getTitle().getLang();
    } else {
      return null;
    }
  }

  @Override
  void setTitleLang(Object depositDocObj, String value) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getTitle() == null) {
      depositDoc.setTitle(new Text());
    }
    depositDoc.getTitle().setLang(value);
  }

  @Override
  public void setISSN(Object depositDocObj, String value) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getContainer() == null) {
      depositDoc.setContainer(new Container());
    }
    if (depositDoc.getContainer().getTitle() == null) {
      depositDoc.getContainer().setTitle(this.factory.createText());
    }
    depositDoc.getContainer().getTitle().setContent(value);
  }

  @Override
  public String getISSN(Object depositDocObj) {
    return this.getContainerISSN(depositDocObj);
  }

  public String getContainerISSN(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getContainer() != null && depositDoc.getContainer().getIdentifiers() != null) {
      return depositDoc.getContainer().getIdentifiers().getIssn();
    } else {
      return null;
    }
  }

  public String getContainerISBN(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getContainer() != null && depositDoc.getContainer().getIdentifiers() != null) {
      return depositDoc.getContainer().getIdentifiers().getIsbn();
    } else {
      return null;
    }
  }

  @Override
  public String getISBN(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getIdentifiers() != null) {
      return depositDoc.getIdentifiers().getIsbn();
    } else {
      return null;
    }
  }

  @Override
  public String getArxivId(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getIdentifiers() != null) {
      return depositDoc.getIdentifiers().getArxiv();
    } else {
      return null;
    }
  }

  @Override
  public String getPmcId(Object depositDocObj) {
    return null;
  }

  @Override
  String getDOI(Object depositDocObj) {
    List<String> dois = this.getDOIs(depositDocObj);
    if (dois != null && !dois.isEmpty()) {
      return dois.get(0);
    }
    return null;
  }

  @Override
  BigInteger getPmid(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getIdentifiers() != null) {
      return depositDoc.getIdentifiers().getPmid();
    } else {
      return null;
    }
  }

  @Override
  public String getURN(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getIdentifiers() != null) {
      return depositDoc.getIdentifiers().getUrn();
    } else {
      return null;
    }
  }

  public List<String> getDOIs(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getIdentifiers() != null && depositDoc.getIdentifiers().getDois() != null) {
      return depositDoc.getIdentifiers().getDois().getDoi();
    } else {
      return null;
    }
  }

  @Override
  public List<AbstractContributor> getContributors(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    List<AbstractContributor> contributors = new ArrayList<>();
    if (depositDoc.getContributors() != null) {
      List<Serializable> contributorOrCollaborations = depositDoc.getContributors().getContributorOrCollaboration();
      for (Serializable serializable : contributorOrCollaborations) {
        if (serializable instanceof Contributor) {
          ContributorDTO contributor = new ContributorDTO();
          contributor.setFirstName(((Contributor) serializable).getFirstname());
          contributor.setLastName(((Contributor) serializable).getLastname());
          contributor.setEmail(((Contributor) serializable).getEmail());
          contributor.setCnIndividu(((Contributor) serializable).getCnIndividu());
          contributor.setInstitution(((Contributor) serializable).getInstitution());
          contributor.setOrcid(((Contributor) serializable).getOrcid());
          if (((Contributor) serializable).getRole() != null) {
            contributor.setRole(((Contributor) serializable).getRole().value());
          }
          contributors.add(contributor);
        } else {
          CollaborationDTO collaboration = new CollaborationDTO();
          collaboration.setName(((Collaboration) serializable).getName());
          contributors.add(collaboration);
        }
      }
    }
    return contributors;
  }

  @Override
  public void setContributors(Object depositDocObj, List<AbstractContributor> contributors) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getContributors() == null) {
      depositDoc.setContributors(this.factory.createContributors());
    }
    depositDoc.getContributors().getContributorOrCollaboration().clear();
    for (AbstractContributor abstractContributor : contributors) {
      if (abstractContributor instanceof ContributorDTO) {
        ContributorDTO contributorDTO = (ContributorDTO) abstractContributor;
        Contributor contributor = this.factory.createContributor();
        contributor.setRole(AuthorRole.fromValue(contributorDTO.getRole()));
        contributor.setFirstname(contributorDTO.getFirstName());
        contributor.setLastname(contributorDTO.getLastName());
        contributor.setEmail(contributorDTO.getEmail());
        contributor.setCnIndividu(contributorDTO.getCnIndividu());
        contributor.setInstitution(contributorDTO.getInstitution());
        contributor.setOrcid(contributorDTO.getOrcid());
        depositDoc.getContributors().getContributorOrCollaboration().add(contributor);
      } else if (abstractContributor instanceof CollaborationDTO) {
        CollaborationDTO collaborationDTO = (CollaborationDTO) abstractContributor;
        Collaboration collaboration = this.factory.createCollaboration();
        collaboration.setName(collaborationDTO.getName());
        depositDoc.getContributors().getContributorOrCollaboration().add(collaboration);
      }
    }
  }

  @Override
  public List<ResearchGroup> getResearchGroups(Object depositDocObj) {
    List<ResearchGroup> researchGroups = new ArrayList<>();
    Groups groups = ((DepositDoc) depositDocObj).getGroups();
    if (groups != null) {
      List<Group> groupList = groups.getGroup();
      for (Group g : groupList) {
        ResearchGroup researchGroup = new ResearchGroup();
        researchGroup.setName((g != null && g.getName() != null) ? g.getName() : null);  // to avoid null exception
        researchGroup.setCode((g != null && g.getCode() != null) ? g.getCode().toString() : null);
        researchGroups.add(researchGroup);
      }
    }
    return researchGroups;
  }

  @Override
  public List<Structure> getStructures(Object depositDocObj) {
    List<Structure> structures = new ArrayList<>();
    AcademicStructures academicStructures = ((DepositDoc) depositDocObj).getAcademicStructures();
    if (academicStructures != null) {
      List<AcademicStructure> academicStructuresList = academicStructures.getAcademicStructure();
      for (AcademicStructure as : academicStructuresList) {
        Structure structure = new Structure();
        structure.setName((as != null && as.getName() != null) ? as.getName() : ""); // to avoid null exception
        structure.setCodeStruct((as != null && as.getCode() != null) ? as.getCode() : "");
        structure.setCnStructC((as != null && as.getOldCode() != null) ? as.getOldCode() : "");
        structures.add(structure);
      }
    }
    return structures;
  }

  @Override
  public List<MetadataDates> getDates(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    List<MetadataDates> metadataDates = new ArrayList<>();
    if (depositDoc.getDates() != null) {
      List<DateWithType> datesWithType = depositDoc.getDates().getDate();
      for (DateWithType date : datesWithType) {
        MetadataDates metadataDate = new MetadataDates();
        metadataDate.setContent(date.getContent());
        metadataDate.setType(date.getType().value());
        metadataDates.add(metadataDate);
      }
    }
    return metadataDates;
  }

  @Override
  String getOriginalTitleContent(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getOriginalTitle() != null) {
      return depositDoc.getOriginalTitle().getContent();
    }
    return null;
  }

  @Override
  String getOriginalTitleLang(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getOriginalTitle() != null) {
      return depositDoc.getOriginalTitle().getLang();
    }
    return null;
  }

  @Override
  void setOriginalTitleLang(Object depositDocObj, String value) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getOriginalTitle() == null) {
      depositDoc.setOriginalTitle(this.factory.createTextLang());
    }
    depositDoc.getOriginalTitle().setLang(value);
  }

  String getContainerTitleLang(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getContainer() != null && depositDoc.getContainer().getTitle() != null) {
      return depositDoc.getContainer().getTitle().getLang();
    }
    return null;
  }

  void setContainerTitleLang(Object depositDocObj, String value) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getContainer() == null) {
      depositDoc.setContainer(this.factory.createContainer());
    }
    if (depositDoc.getContainer().getTitle() == null) {
      depositDoc.getContainer().setTitle(this.factory.createText());
    }
    depositDoc.getContainer().getTitle().setLang(value);
  }

  @Override
  List<MetadataTextLang> getAbstracts(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    List<MetadataTextLang> abstracts = new ArrayList<>();
    if (depositDoc.getAbstracts() != null && depositDoc.getAbstracts().getAbstract() != null) {
      for (Text abstractText : depositDoc.getAbstracts().getAbstract()) {
        abstracts.add(new MetadataTextLang(abstractText.getContent(), abstractText.getLang()));
      }
    }
    return abstracts;
  }

  @Override
  void setAbstracts(Object depositDocObj, List<MetadataTextLang> abstracts) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getAbstracts() == null) {
      depositDoc.setAbstracts(this.factory.createAbstracts());
    }
    depositDoc.getAbstracts().getAbstract().clear();
    for (MetadataTextLang textLang : abstracts) {
      Text text = this.factory.createText();
      text.setContent(textLang.getContent());
      text.setLang(textLang.getLang());
      depositDoc.getAbstracts().getAbstract().add(text);
    }
  }

  List<MetadataTextLang> getCitations(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getCitations() != null && depositDoc.getCitations().getCitation() != null) {
      List<MetadataTextLang> textLangs = new ArrayList<>();
      for (Text abstractText : depositDoc.getCitations().getCitation()) {
        textLangs.add(new MetadataTextLang(abstractText.getContent(), abstractText.getLang()));
      }
      return textLangs;
    }
    return null;
  }

  @Override
  List<MetadataCollection> getCollections(Object depositDocObj) {
    ch.unige.aou.model.xml.deposit.v1.DepositDoc depositDoc = (ch.unige.aou.model.xml.deposit.v1.DepositDoc) depositDocObj;
    List<MetadataCollection> collections = new ArrayList<>();
    if (depositDoc.getCollections() != null && depositDoc.getCollections().getCollection() != null) {
      for (ch.unige.aou.model.xml.deposit.v1.Collection collection : depositDoc.getCollections().getCollection()) {
        collections.add(new MetadataCollection(collection.getName(), collection.getNumber().toString()));
      }
    }
    return collections;
  }

  void setCitations(Object depositDocObj, List<MetadataTextLang> citations) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getCitations() == null) {
      depositDoc.setCitations(this.factory.createCitations());
    }
    depositDoc.getCitations().getCitation().clear();
    for (MetadataTextLang textLang : citations) {
      Text text = this.factory.createText();
      text.setContent(textLang.getContent());
      text.setLang(textLang.getLang());
      depositDoc.getCitations().getCitation().add(text);
    }
  }

  @Override
  List<MetadataLink> getLinks(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    List<MetadataLink> metadataLinks = new ArrayList<>();
    if (depositDoc.getLinks() != null && depositDoc.getLinks().getLink() != null) {
      for (Link link : depositDoc.getLinks().getLink()) {
        MetadataLink metadataLink = new MetadataLink();
        metadataLink.setTarget(link.getTarget());
        if (link.getDescription() != null) {
          metadataLink.setDescription(new MetadataTextLang(link.getDescription().getContent(), link.getDescription().getLang()));
        }
        if (link.getType() == LinkTypes.WEB) {
          metadataLink.setType(MetadataLink.LinkType.WEB);
        } else if (link.getType() == LinkTypes.INTERNAL) {
          metadataLink.setType(MetadataLink.LinkType.INTERNAL);
        }
        metadataLinks.add(metadataLink);
      }
    }
    return metadataLinks;
  }

  @Override
  void setLinks(Object depositDocObj, List<MetadataLink> links) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getLinks() == null) {
      depositDoc.setLinks(this.factory.createLinks());
    }
    depositDoc.getLinks().getLink().clear();
    for (MetadataLink metadataLink : links) {
      Link link = this.factory.createLink();
      link.setTarget(metadataLink.getTarget());

      if (metadataLink.getDescription() != null) {
        Text text = this.factory.createText();
        text.setContent(metadataLink.getDescription().getContent());
        text.setLang(metadataLink.getDescription().getLang());
        link.setDescription(text);
      }

      if (metadataLink.getType() == MetadataLink.LinkType.WEB) {
        link.setType(LinkTypes.WEB);
      } else if (link.getType() == LinkTypes.INTERNAL) {
        link.setType(LinkTypes.INTERNAL);
      }
      depositDoc.getLinks().getLink().add(link);
    }
  }

  @Override
  public List<String> getLanguages(Object depositDocObj) {
    ch.unige.aou.model.xml.deposit.v2.DepositDoc depositDoc = (ch.unige.aou.model.xml.deposit.v2.DepositDoc) depositDocObj;
    if (depositDoc.getLanguages() != null) {
      return depositDoc.getLanguages().getLanguage();
    } else {
      return new ArrayList<>();
    }
  }

  @Override
  public List<MetadataFunding> getFundings(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    List<MetadataFunding> metadataFundings = new ArrayList<>();
    if (depositDoc.getFundings() != null && depositDoc.getFundings().getFunding() != null) {
      for (Funding funding : depositDoc.getFundings().getFunding()) {
        MetadataFunding metadataFunding = new MetadataFunding();
        metadataFunding.setFunder(funding.getFunder());
        metadataFunding.setName(funding.getName());
        metadataFunding.setCode(String.valueOf(funding.getCode()));
        metadataFunding.setAcronym(funding.getAcronym());
        metadataFundings.add(metadataFunding);
      }
    }
    return metadataFundings;
  }

  @Override
  List<String> getDatasets(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    List<String> datasets = new ArrayList<>();
    if (depositDoc.getDataset() != null && depositDoc.getDataset().getUrlOrDoi() != null) {
      for (JAXBElement<String> jaxbElement : depositDoc.getDataset().getUrlOrDoi()) {
        datasets.add(jaxbElement.getValue() != null ? jaxbElement.getValue() : null);
      }
    }
    return datasets;
  }

  @Override
  List<MetadataFile> getFiles(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    List<MetadataFile> files = new ArrayList<>();
    if (depositDoc.getFiles() != null && depositDoc.getFiles().getFile() != null) {
      for (File file : depositDoc.getFiles().getFile()) {
        MetadataFile metadataFile = new MetadataFile();
        metadataFile.setFileTypeValue(file.getType().value());
        metadataFile.setName(file.getName());
        metadataFile.setSize(file.getSize());
        metadataFile.setMimeType(file.getMimetype());
        metadataFile.setAccessLevel(file.getAccessLevel().value());
        metadataFile.setLicense(file.getLicense());
        files.add(metadataFile);
      }
    }
    return files;
  }

  @Override
  List<MetadataCorrection> getCorrections(Object depositDocObj) {
    // V1 doesn't have any correction
    return new ArrayList<>();
  }

  /*******************************************************************************************/

  private ObjectMapper getJsonFormDeserializerMapper() {
    SimpleModule module = new SimpleModule();
    module.addDeserializer(Abstracts.class, new AbstractsDeserializer());
    module.addDeserializer(AcademicStructures.class, new AcademicsStructuresDeserializer());
    module.addDeserializer(Citations.class, new CitationsDeserializer());
    module.addDeserializer(Classifications.class, new ClassificationsDeserializer());
    module.addDeserializer(Collections.class, new CollectionsDeserializer());
    module.addDeserializer(Container.class, new ContainerDeserializer());
    module.addDeserializer(ContainerIdentifiers.class, new ContainerIdentifiersDeserializer());
    module.addDeserializer(Contributors.class, new ContributorsDeserializer());
    module.addDeserializer(Dataset.class, new DatasetDeserializer());
    module.addDeserializer(Dates.class, new DatesDeserializer());
    module.addDeserializer(DateWithType.class, new DateWithTypeDeserializer());
    module.addDeserializer(DepositIdentifiers.class, new DepositIdentifiersDeserializer());
    module.addDeserializer(Dois.class, new DoisDeserializer());
    module.addDeserializer(Files.class, new FilesDeserializer());
    module.addDeserializer(Fundings.class, new FundingsDeserializer());
    module.addDeserializer(Groups.class, new GroupsDeserializer());
    module.addDeserializer(Keywords.class, new KeywordsDeserializer());
    module.addDeserializer(Links.class, new LinksDeserializer());
    module.addDeserializer(Pages.class, new PagesDeserializer());
    module.addDeserializer(Publisher.class, new PublisherDeserializer());
    module.addDeserializer(Rights.class, new RightsDeserializer());
    module.addDeserializer(TextLang.class, new TextLangDeserializer());
    module.addDeserializer(Text.class, new TextDeserializer());

    ObjectMapper mapper = new ObjectMapper();
    mapper.registerModule(module);
    mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
    return mapper;
  }

  private ObjectMapper getJsonFormSerializerMapper() {
    SimpleModule module = new SimpleModule();
    module.addSerializer(Abstracts.class, new AbstractsSerializer());
    module.addSerializer(AcademicStructures.class, new AcademicStructuresSerializer());
    module.addSerializer(AuthorRole.class, new AuthorRoleSerializer());
    module.addSerializer(Citations.class, new CitationsSerializer());
    module.addSerializer(Classifications.class, new ClassificationsSerializer());
    module.addSerializer(Collections.class, new CollectionsSerializer());
    module.addSerializer(ContainerIdentifiers.class, new ContainerIdentifiersSerializer());
    module.addSerializer(Contributors.class, new ContributorsSerializer());
    module.addSerializer(Dataset.class, new DatasetSerializer());
    module.addSerializer(Dates.class, new DatesSerializer());
    module.addSerializer(DateWithType.class, new DateWithTypeSerializer());
    module.addSerializer(DepositDoc.class, new DepositDocSerializer());
    module.addSerializer(DepositIdentifiers.class, new DepositIdentifiersSerializer());
    module.addSerializer(Dois.class, new DoisSerializer());
    module.addSerializer(Files.class, new FilesSerializer());
    module.addSerializer(Fundings.class, new FundingsSerializer());
    module.addSerializer(Groups.class, new GroupsSerializer());
    module.addSerializer(Keywords.class, new KeywordsSerializer());
    module.addSerializer(Links.class, new LinksSerializer());
    module.addSerializer(Pages.class, new PagesSerializer());
    module.addSerializer(Publisher.class, new PublisherSerializer());
    module.addSerializer(Rights.class, new RightsSerializer());
    module.addSerializer(Text.class, new TextSerializer());
    module.addSerializer(TextLang.class, new TextLangSerializer());

    ObjectMapper mapper = new ObjectMapper();
    mapper.registerModule(module);
    mapper.enable(SerializationFeature.INDENT_OUTPUT);
    return mapper;
  }

  private DepositDoc getDepositDoc(Publication publication) {
    return (DepositDoc) this.createDepositDocObjectFromXml(publication.getMetadata());
  }

  private File convertDocumentFileToXmlFile(DocumentFile documentFile) {
    try {
      File file = new File();
      file.setType(FileType.fromValue(documentFile.getDocumentFileType().getValue()));
      file.setAccessLevel(AccessLevel.fromValue(
              documentFile.getAccessLevel().toString().substring(0, 1).toUpperCase() + documentFile.getAccessLevel().toString().substring(1)
                      .toLowerCase()));
      file.setName(documentFile.getFileName());
      file.setSize(BigInteger.valueOf(documentFile.getFileSize()));
      file.setMimetype(documentFile.getMimetype());
      if (documentFile.getLicense() != null) {
        file.setLicense(documentFile.getLicense().getOpenLicenseId());
      }
      if (documentFile.getEmbargoEndDate() != null) {
        Embargo embargo = new Embargo();
        embargo.setAccessLevel(AccessLevel.fromValue(
                documentFile.getEmbargoAccessLevel().toString().substring(0, 1).toUpperCase() + documentFile.getEmbargoAccessLevel().toString()
                        .substring(1).toLowerCase()));
        embargo.setEndDate(DatatypeFactory.newInstance().newXMLGregorianCalendar(documentFile.getEmbargoEndDate().toString()));
        file.setEmbargo(embargo);
      }
      return file;
    } catch (DatatypeConfigurationException e) {
      throw new SolidifyResourceNotFoundException(
              "'Unable to convert embargo date'" + documentFile.getEmbargoEndDate() + "'to XMLGregorianCalendar format'");
    }
  }
}
