package ch.unige.aou.service.rest;

import static java.util.stream.Collectors.counting;
import static java.util.stream.Collectors.groupingBy;

import java.util.List;
import java.util.Map;

import org.springframework.data.domain.Pageable;

import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.rest.RestCollection;
import ch.unige.solidify.util.StringTool;

import ch.unige.aou.model.rest.UnigePersonDTO;

public abstract class UnigePersonSearchService implements UnigePersonSearch {

  private final static String NAME_SEPARATOR = "___";

  @Override
  public RestCollection<UnigePersonDTO> search(String firstname, String lastname, Pageable pageable) {

    if (!StringTool.isNullOrEmpty(lastname)) {
      RestCollection<UnigePersonDTO> results = this.getResultsFromExternalService(firstname, lastname, pageable);
      this.discardNotNecessaryYearsOfBirth(results);
      return results;
    } else {
      throw new SolidifyRuntimeException("lastname is mandatory to search for a person");
    }
  }

  /**
   * Years of birth may be used to differentiate homonyms. If a person name appears only once in the list,
   * the info is not necessary and is removed from results for privacy considerations.
   *
   * @param results
   */
  protected void discardNotNecessaryYearsOfBirth(RestCollection<UnigePersonDTO> results) {
    List<UnigePersonDTO> unigePersonDTOs = results.getData();
    Map<String, Long> fullnameOccurrences = unigePersonDTOs.stream()
            .collect(groupingBy(unigePersonDTO -> unigePersonDTO.getFirstname() + NAME_SEPARATOR + unigePersonDTO.getLastname(), counting()));

    unigePersonDTOs.forEach(unigePersonDTO -> {
      String fullname = unigePersonDTO.getFirstname() + NAME_SEPARATOR + unigePersonDTO.getLastname();
      if (fullnameOccurrences.get(fullname) == 1) {
        unigePersonDTO.setYearOfBirth(null);
      }
    });
  }

  protected abstract RestCollection<UnigePersonDTO> getResultsFromExternalService(String firstname, String lastname, Pageable pageable);
}
