package ch.unige.aou.service.rest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.util.UriComponentsBuilder;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.exception.SolidifyResourceNotFoundException;
import ch.unige.solidify.service.RemoteResourceService;
import ch.unige.solidify.service.SolidifyRestClientService;
import ch.unige.solidify.util.PropagateRestClientTool;

import ch.unige.aou.config.AouProperties;
import ch.unige.aou.model.security.User;
import ch.unige.aou.model.security.UserDisplayableInfos;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.ResourceName;

@Service
public class UserRemoteResourceService extends RemoteResourceService<User> {

  private static final Logger log = LoggerFactory.getLogger(UserRemoteResourceService.class);

  private final PropagateRestClientTool restClientTool;

  final private String adminModuleUrl;

  public UserRemoteResourceService(SolidifyRestClientService restClientService, AouProperties aouProperties,
          PropagateRestClientTool restClientTool) {
    super(restClientService);
    this.restClientTool = restClientTool;

    this.adminModuleUrl = aouProperties.getModule().getAdmin().getUrl();
  }

  public User getAuthenticatedUser() {
    String url = UriComponentsBuilder.fromUriString(
            this.adminModuleUrl + SolidifyConstants.URL_SEP + ResourceName.USERS + SolidifyConstants.URL_SEP + AouActionName.AUTHENTICATED)
            .toUriString();
    try {
      final RestTemplate restClient = this.restClientTool.getClient();
      return restClient.getForObject(url, User.class);
    } catch (HttpClientErrorException.NotFound e) {
      throw new SolidifyResourceNotFoundException(url);
    }
  }

  public UserDisplayableInfos getUserDisplayableInfos(String externalUid) {
    String url = UriComponentsBuilder.fromUriString(
            this.adminModuleUrl + SolidifyConstants.URL_SEP + ResourceName.USERS + SolidifyConstants.URL_SEP
                    + AouActionName.GET_USER_DISPLAYABLE_INFOS + SolidifyConstants.URL_SEP + externalUid).toUriString();
    try {
      final RestTemplate restClient = this.restClientTool.getClient();
      return restClient.getForObject(url, UserDisplayableInfos.class);
    } catch (HttpClientErrorException.NotFound e) {
      throw new SolidifyResourceNotFoundException(url);
    }
  }

  @Override
  protected Class<User> getResourceClass() {
    return User.class;
  }

  @Override
  protected String getResourceUrl() {
    return this.adminModuleUrl + SolidifyConstants.URL_SEP + ResourceName.USERS;
  }
}
