package ch.unige.aou.service;

import java.util.Map;

import org.springframework.stereotype.Service;

import ch.unige.solidify.auth.model.AuthApplicationRole;

import ch.unige.aou.business.PersonService;
import ch.unige.aou.business.UserService;
import ch.unige.aou.model.security.User;

@Service("userPermissionService")
public class UserPermissionService extends AbstractPermissionService {

  private UserService userService;

  public UserPermissionService(PersonService personService, UserService userService) {
    super(personService);
    this.userService = userService;
  }

  public boolean isAllowedToUpdate(String userId, Map<String, Object> updateMap) {
    if (this.hasUserApplicationRole()) {
      // a USER cannot update users
      return false;
    }
    if (this.hasApplicationRole(AuthApplicationRole.ROOT) || this.hasApplicationRole(AuthApplicationRole.TRUSTED_CLIENT)) {
      // a ROOT or TRUSTED_CLIENT can update all users
      return true;
    }
    if (this.hasApplicationRole(AuthApplicationRole.ADMIN)) {
      User beforeUpdateUser = this.userService.findOne(userId);
      final String beforeUpdateApplicationRoleId = beforeUpdateUser.getApplicationRole().getResId();
      final String afterUpdateApplicationRoleId = this.userService.getApplicationRoleId(updateMap);

      if (!this.userService.containsApplicationRoleId(updateMap) || beforeUpdateApplicationRoleId.equals(afterUpdateApplicationRoleId)) {
        // ADMIN can update other users if they don't change their roles
        return true;
      } else if (AuthApplicationRole.ROOT.equals(beforeUpdateUser.getApplicationRole())) {
        // ADMIN cannot update the role of ROOT users
        return false;
      } else if (AuthApplicationRole.ROOT.getResId().equals(afterUpdateApplicationRoleId)) {
        // ADMIN cannot make other users ROOT
        return false;
      }
    }
    return false;
  }

  public boolean hasValidationRights() {
    String personId = this.personService.getLinkedPersonId(this.getAuthentication());
    return this.personService.hasValidationRights(personId);
  }
}
