package ch.unige.aou.test.resource;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;

import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.unige.aou.rest.ResourceName;
import ch.unige.aou.rest.UrlPath;

public class ResearchGroupDoc extends AbstractResourceDoc {
  @Override
  public List<String> getListLinks() {
    return Collections.emptyList();
  }

  @Override
  public String getName() {
    return UrlPath.ADMIN_RESEARCH_GROUPS;
  }

  @Override
  public List<String> getResourceLinks() {
    return Arrays.asList(ResourceName.PEOPLE, ResourceName.PUBLICATIONS);
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    // @formatter:off
    return Arrays.asList(
            new FieldDescriptorBuilder(this)
               .name("name")
               .description("The name of the research group")
               .mandatory(true)
               .type(TypeField.STRING)
               .build(),
            new FieldDescriptorBuilder(this)
              .name("code")
              .description("The code of the research group")
              .mandatory(true)
              .type(TypeField.INT32)
              .testValue("100")
              .build(),
            new FieldDescriptorBuilder(this)
              .name("acronym")
              .description("The acronym of the research group")
              .mandatory(true)
              .type(TypeField.STRING)
              .testValue("GBD")
              .build(),
            new FieldDescriptorBuilder(this)
              .name("active")
              .description("Indicate if the research group is still active")
              .mandatory(true)
              .type(TypeField.BOOLEAN)
              .testValue("true")
              .build(),
            new FieldDescriptorBuilder(this)
              .name("validated")
              .description("Indicate if the research group is validated")
              .mandatory(true)
              .type(TypeField.BOOLEAN)
              .testValue("true")
              .build()
    );
    // @formatter:on
  }
}
